@@ -1,9 +1,9 @@
 /*
- * $Id: fcc_data.c,v 1.10 2004/08/19 06:38:23 we7u Exp $
+ * $Id: fcc_data.c,v 1.13 2005/02/04 19:36:06 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2004  The Xastir Group
+ * Copyright (C) 2000-2005  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -102,6 +102,7 @@ int build_fcc_index(int type){
             return(1);
         else {
             statusline("FCC index old, rebuilding",1);
+            fprintf(stderr,"FCC index is old.  Rebuilding index.\n");
 //            XmTextFieldSetString(text,"FCC index old, rebuilding");
 //            XtManageChild(text);
 //            XmUpdateDisplay(XtParent(text));     // DK7IN: do we need this ???
@@ -155,6 +156,7 @@ int build_fcc_index(int type){
 
 //    XmTextFieldSetString(text,"");
 //    XtManageChild(text);
+
     return(1);
 }
 
@@ -463,7 +465,7 @@ int search_fcc_data_appl(char *callsign, FccAppl *data) {
                             // location for the callsign.  Return if so.
                             if ( (temp[0] < line[pos]) ||
                                     ( (temp[0] == line[pos]) && (temp[1] < line[pos+1]) ) ) {
-                                popup_message("Callsign Search", "Callsign Not Found!");
+                                popup_message_always("Callsign Search", "Callsign Not Found!");
                                 //fprintf(stderr,"%c%c\t%c%c\n",temp[0],temp[1],line[pos],line[pos+1]);
                                 (void)fclose(f);
                                 return(0);
