@@ -1,9 +1,9 @@
 /*
- * $Id: geocoder_gui.c,v 1.10 2004/08/21 06:13:58 we7u Exp $
+ * $Id: geocoder_gui.c,v 1.15 2005/07/21 20:04:40 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2004  The Xastir Group
+ * Copyright (C) 2000-2005  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -146,6 +146,15 @@ void Geocoder_place_now(Widget w, XtPointer clientData, XtPointer callData) {
     (void)remove_trailing_spaces(geocoder_locality_name);
     (void)remove_trailing_spaces(geocoder_address_name);
 
+/*
+fprintf(stderr,"%s\n%s\n%s\n%s\n%s\n",
+    geocoder_zip_name,
+    geocoder_state_name,
+    geocoder_locality_name,
+    geocoder_address_name,
+    geocoder_map_filename);
+*/
+
     index = io_open(geocoder_map_filename);
 
     xastir_snprintf(input,
@@ -157,6 +166,9 @@ void Geocoder_place_now(Widget w, XtPointer clientData, XtPointer callData) {
         geocoder_state_name,
         geocoder_zip_name);
 
+    if (debug_level & 1)
+        fprintf(stderr,"Searching for: %s\n", input);
+
     if (geo_find(index,input,strlen(input),&loc)) {
         long coord_lon, coord_lat;
         char lat_str[20];
@@ -166,6 +178,7 @@ void Geocoder_place_now(Widget w, XtPointer clientData, XtPointer callData) {
         char latd = 'N';
         double res, tmp;
 
+
         if (loc.at.longitude < 0) {
             loc.at.longitude = -loc.at.longitude;
             lond = 'W';
@@ -215,11 +228,11 @@ void Geocoder_place_now(Widget w, XtPointer clientData, XtPointer callData) {
         destination_coord_lon = coord_lon;
         mark_destination = 1;
 
-        popup_message( langcode("POPEM00029"), geocoder_address_name );
+        popup_message_always( langcode("POPEM00029"), geocoder_address_name );
         set_map_position(w, coord_lat, coord_lon);
     }
     else {
-        popup_message(langcode("POPEM00025"),geocoder_address_name);
+        popup_message_always(langcode("POPEM00025"),geocoder_address_name);
     }
     Geocoder_place_destroy_shell(w, clientData, callData);
 }
@@ -253,7 +266,7 @@ void Geocoder_place(/*@unused@*/ Widget w, /*@unused@*/ XtPointer clientData, /*
 
 begin_critical_section(&geocoder_place_dialog_lock, "geocoder_gui.c:Geocoder_place" );
 
-        geocoder_place_dialog = XtVaCreatePopupShell(langcode("PULDNMP014"),
+        geocoder_place_dialog = XtVaCreatePopupShell(langcode("PULDNMP029"),
                 xmDialogShellWidgetClass,Global.top,
                 XmNdeleteResponse,XmDESTROY,
                 XmNdefaultPosition, FALSE,
@@ -394,7 +407,6 @@ begin_critical_section(&geocoder_place_dialog_lock, "geocoder_gui.c:Geocoder_pla
 
 //        zip = XtVaCreateManagedWidget(langcode("FEATURE001"),xmLabelWidgetClass, form,
         zip = XtVaCreateManagedWidget("Zip Code:",xmLabelWidgetClass, form,
- 
                 XmNtopAttachment, XmATTACH_WIDGET,
                 XmNtopWidget, state,
                 XmNtopOffset, 10,
