@@ -1,9 +1,9 @@
 /*
- * $Id: gps.c,v 1.43 2004/08/21 06:13:58 we7u Exp $
+ * $Id: gps.c,v 1.48 2005/06/01 19:24:44 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2004  The Xastir Group
+ * Copyright (C) 2000-2005  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -73,6 +73,26 @@ int gps_stop_now;
 
 
 // This function is destructive to its first parameter
+//
+// GPRMC,UTC-Time,status(A/V),lat,N/S,lon,E/W,SOG,COG,UTC-Date,Mag-Var,E/W[*CHK]
+// GPRMC,hhmmss[.sss],{A|V},ddmm.mm[mm],{N|S},dddmm.mm[mm],{E|W},[dd]d.d[ddddd],[dd]d.d[d],ddmmyy,[ddd.d],[{E|W}][,A|D|E|N|S][*CHK]
+//
+// The last field before the checksum is entirely optional, and in
+// fact first appeared in NMEA 2.3 (fairly recently).  Most GPS's do
+// not currently put out that field.  The field may be null or
+// nonexistent including the comma.  Only "A" or "D" are considered
+// to be active and reliable fixes if this field is present.
+// Fix-Quality:
+//  A: Autonomous
+//  D: Differential
+//  E: Estimated
+//  N: Not Valid
+//  S: Simulator
+//
+// $GPRMC,081836,A,3751.65,S,14507.36,E,000.0,360.0,130998,011.3,E*62
+// $GPRMC,104748.821,A,4301.1492,N,08803.0374,W,0.085048,102.36,010605,,*1A
+// $GPRMC,104749.821,A,4301.1492,N,08803.0377,W,0.054215,74.60,010605,,*2D
+//
 int decode_gps_rmc( char *data,
                     char *long_pos,
                     int long_pos_length,
@@ -86,8 +106,9 @@ int decode_gps_rmc( char *data,
                     int *status) {
 
     char *temp_ptr;
+    char *temp_ptr2;
     char temp_data[MAX_TNC_LINE_SIZE+1];    // Big in case we get concatenated packets (it happens!)
-    char sampletime[7];
+    char sampletime[7]; // We ignore fractional seconds
     char long_pos_x[11];
     char long_ew;
     char lat_pos_y[10];
@@ -149,9 +170,25 @@ int decode_gps_rmc( char *data,
 
     temp_ptr=strtok(NULL,",");  // get latitude
 
-    if (temp_ptr == NULL || temp_ptr[4] != '.')
+    if (temp_ptr == NULL)
         return(0);  // Doesn't look like latitude
 
+    // Newer GPS'es appear not to zero-fill on the left.  Check for
+    // the decimal point in all the possible places.
+    if (temp_ptr[1] != '.'
+            && temp_ptr[2] != '.'
+            && temp_ptr[3] != '.'
+            && temp_ptr[4] != '.') {
+        return(0);  // Doesn't look like latitude
+    }
+
+// Note:  Starlink Invicta shows "lllll.ll" format for latitude in
+// the GPRMC sentence, which would mean we'd need another term in
+// the above, and would need to terminate at [10] below (making sure
+// we extended the field another char as well to handle it).  I'm
+// hoping it was a typo in the Starlink Invicta spec, as latitude
+// never requires three digits for degrees.
+
     xastir_snprintf(lat_pos_y,
         sizeof(lat_pos_y),
         "%s",
@@ -160,9 +197,10 @@ int decode_gps_rmc( char *data,
 
 // Note that some GPS's put out latitude with extra precision, such as 4801.1234
 
-    // Check for comma char
-    if (lat_pos_y[8] == ',')
-        lat_pos_y[8] = '\0';
+    // Check for comma char, replace with '\0'
+    temp_ptr2 = strstr(lat_pos_y, ",");
+    if (temp_ptr2)
+        temp_ptr2[0] = '\0';
 
     temp_ptr=strtok(NULL,",");  // get N-S
 
@@ -182,9 +220,20 @@ int decode_gps_rmc( char *data,
 
     temp_ptr=strtok(NULL,",");  // get long
 
-    if (temp_ptr == NULL || temp_ptr[5] != '.')
+    if (temp_ptr == NULL)
         return(0);  // Doesn't look like longitude
 
+    // Newer GPS'es appear not to zero-fill on the left.  Check for
+    // the decimal point in all the possible places.
+    if (temp_ptr[1] != '.'
+            && temp_ptr[2] != '.'
+            && temp_ptr[3] != '.'
+            && temp_ptr[4] != '.'
+            && temp_ptr[5] != '.') {
+        return(0);  // Doesn't look like longitude
+    }
+
+
     xastir_snprintf(long_pos_x,
         sizeof(long_pos_x),
         "%s",
@@ -193,9 +242,10 @@ int decode_gps_rmc( char *data,
 
 // Note that some GPS's put out longitude with extra precision, such as 12201.1234
 
-    // Check for comma char
-    if (long_pos_x[9] == ',')
-        long_pos_x[9] = '\0';
+    // Check for comma char, replace with '\0'
+    temp_ptr2 = strstr(long_pos_x, ",");
+    if (temp_ptr2)
+        temp_ptr2[0] = '\0';
 
     temp_ptr=strtok(NULL,",");  // get E-W
 
@@ -289,6 +339,24 @@ int decode_gps_rmc( char *data,
 
 
 // This function is destructive to its first parameter
+//
+// GPGGA,UTC-Time,lat,N/S,long,E/W,GPS-Quality,nsat,HDOP,MSL-Meters,M,Geoidal-Meters,M,DGPS-Data-Age(seconds),DGPS-Ref-Station-ID[*CHK]
+// GPGGA,hhmmss[.sss],ddmm.mm[mm],{N|S},dddmm.mm[mm],{E|W},{0-8},dd,[d]d.d,[-dddd]d.d,M,[-ddd]d.d,M,[dddd.d],[dddd][*CHK]
+//
+// GPS-Quality:
+//  0: Invalid Fix
+//  1: GPS Fix
+//  2: DGPS Fix
+//  3: PPS Fix
+//  4: RTK Fix
+//  5: Float RTK Fix
+//  6: Estimated (dead-reckoning) Fix
+//  7: Manual Input Mode
+//  8: Simulation Mode
+//
+// $GPGGA,170834,4124.8963,N,08151.6838,W,1,05,1.5,280.2,M,-34.0,M,,,*75 
+// $GPGGA,104438.833,4301.1439,N,08803.0338,W,1,05,1.8,185.8,M,-34.2,M,0.0,0000*40
+//
 int decode_gps_gga( char *data,
                     char *long_pos,
                     int long_pos_length,
@@ -300,6 +368,7 @@ int decode_gps_gga( char *data,
                     int *status ) {
 
     char *temp_ptr;
+    char *temp_ptr2;
     char temp_data[MAX_TNC_LINE_SIZE+1];    // Big in case we get concatenated packets (it happens!)
     char long_pos_x[11];
     char long_ew;
@@ -342,9 +411,10 @@ int decode_gps_gga( char *data,
 
 // Note that some GPS's put out latitude with extra precision, such as 4801.1234
 
-    // Check for comma char
-    if (lat_pos_y[8] == ',')
-        lat_pos_y[8] = '\0';
+    // Check for comma char, replace with '\0'
+    temp_ptr2 = strstr(lat_pos_y, ",");
+    if (temp_ptr2)
+        temp_ptr2[0] = '\0';
 
     temp_ptr = strtok(NULL,",");    // get N-S
 
@@ -375,9 +445,10 @@ int decode_gps_gga( char *data,
 
 // Note that some GPS's put out longitude with extra precision, such as 12201.1234
 
-    // Check for comma char
-    if (long_pos_x[9] == ',')
-        long_pos_x[9] = '\0';
+    // Check for comma char, replace with '\0'
+    temp_ptr2 = strstr(long_pos_x, ",");
+    if (temp_ptr2)
+        temp_ptr2[0] = '\0';
 
     temp_ptr = strtok(NULL,",");    // get E-W
 
@@ -406,7 +477,8 @@ int decode_gps_gga( char *data,
         temp_ptr);
     temp_data[1] = '\0';
 
-    if(temp_data[0] != '1' && temp_data[0] != '2' )
+    // '0' = bad fix, positive numbers = ok
+    if(temp_data[0] == '0')
         return(0);
 
     // Save the fix quality in "status"
@@ -476,10 +548,13 @@ int gps_data_find(char *gps_line_data, int port) {
 
     char long_pos[20],lat_pos[20],aunit[2];
     time_t t;
-    struct timeval tv;
-    struct timezone tz;
     char temp_str[MAX_GPS_STRING+1];
     int have_valid_string = 0;
+#ifndef __CYGWIN__
+    struct timeval tv;
+    struct timezone tz;
+#endif  // __CYGWIN__
+ 
 
 
     if (strncmp(gps_line_data,"$GPRMC,",7)==0) {
@@ -530,6 +605,11 @@ int gps_data_find(char *gps_line_data, int port) {
                     port, devices[port].set_time);
             }
 
+// Don't set the time if it's a Cygwin system.  Causes problems with
+// date, plus time can be an hour off if daylight savings time is
+// enabled on Windows.
+//
+#ifndef __CYGWIN__
             if (devices[port].set_time) {
                 tv.tv_sec=t;
                 tv.tv_usec=0;
@@ -540,6 +620,7 @@ int gps_data_find(char *gps_line_data, int port) {
                     fprintf(stderr,"Setting Time %ld EUID: %d, RUID: %d\n",
                         (long)t, (int)getuid(), (int)getuid());
                 }
+
 #ifdef HAVE_SETTIMEOFDAY
 
 ENABLE_SETUID_PRIVILEGE;
@@ -547,7 +628,11 @@ ENABLE_SETUID_PRIVILEGE;
 DISABLE_SETUID_PRIVILEGE;
 
 #endif  // HAVE_SETTIMEOFDAY
+
             }
+
+#endif  // __CYGWIN__
+
         }
     }
     else {
