@@ -1,9 +1,9 @@
 /*
- * $Id: igate.c,v 1.39 2004/08/21 06:13:58 we7u Exp $
+ * $Id: igate.c,v 1.47 2005/03/16 21:36:11 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2004  The Xastir Group
+ * Copyright (C) 2000-2005  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -52,6 +52,10 @@ int max_NWS_stations = 0;
 NWS_Data *NWS_station_data;
 
 
+void load_NWS_stations(char *file);
+int check_NWS_stations(char *call);
+
+
 
 // Struct for holding packet data.  We use dynamically-allocated
 // singly-linked lists.  The last record should have its "next"
@@ -490,6 +494,11 @@ void output_igate_net(char *line, int port, int third_party) {
     if (line[0] == '\0')
         return;
 
+    // Don't igate packets read in from a log file (port -1).
+    // Packets from x_spider (port -2) are ok to igate.
+    if (port == -1)
+        return;
+
 //fprintf(stderr,"Igating: %s\n", line);
 
     // Should we Igate from RF->NET?
@@ -636,14 +645,31 @@ void output_igate_net(char *line, int port, int third_party) {
 
 begin_critical_section(&devices_lock, "igate.c:output_igate_net" );
 
-    // If received from x_spider port or it's our own tactical call
-    if (port < -1)
-        igate_options = 0;
-    else if (port == -1)    // From x_spider server port
-        igate_options = 1;
-    else
+    // If received from x_spider port or it's our own tactical call.
+    // Here are the special port numbers we might see:
+    // -1: We're reading in from a log file
+    // -2: Packet came from x_spider server port (therefore it's
+    //     already authenticated)
+    // -3: We're reading in tactical calls from file
+    //
+    if (port == -1)         // Packet came from a log file.
+        igate_options = 0;  // Don't igate it.
+
+    else if (port == -2)    // Packet came from x_spider server port
+        igate_options = 1;  // Ok to igate.
+
+    else if (port == -3)    // We're reading tactical call from file.
+        igate_options = 0;  // Don't igate it.
+
+    else if (port < -2)     // Errant port number.
+        igate_options = 0;  // Don't igate it.
+
+    else    // Port number is 0 or positive number.  A real port.
+            // Decide whether to igate it based on the port's
+            // configuration.
         igate_options = devices[port].igate_options;
 
+
 end_critical_section(&devices_lock, "igate.c:output_igate_net" );
 
     if (igate_options <= 0 ) {
@@ -733,12 +759,14 @@ end_critical_section(&devices_lock, "igate.c:output_igate_net" );
 
 /****************************************************************/
 /* output data to tnc interfaces                                */
-/* from: type of port heard from                                */
-/* call: call sign heard from                                   */
+/* from: type of port heard from (No! It's the source call!)    */
+/* call: call sign heard from (No! It's the destination call!)  */
 /* line: data to gate to rf                                     */
 /* port: port data came from                                    */
 /****************************************************************/
-void output_igate_rf(char *from, char *call, char *path, char *line, int port, int third_party) {
+void output_igate_rf(char *from, char *call, char *path, char *line,
+        int port, int third_party, char *object_name) {
+
     char temp[MAX_LINE_SIZE+20];
     int x;
     int first = 1;
@@ -826,11 +854,22 @@ void output_igate_rf(char *from, char *call, char *path, char *line, int port, i
         }
         return;
     }
- 
+
+
+     // check to see if the nws-stations file is newer than last read
+    if (last_nws_stations_file_time < file_time(get_user_base_dir("data/nws-stations.txt"))) {
+        last_nws_stations_file_time = file_time(get_user_base_dir("data/nws-stations.txt"));
+        load_NWS_stations(get_user_base_dir("data/nws-stations.txt"));
+        //fprintf(stderr,"NWS Station file time is old\n");
+    }
+
+
     // Check whether the source and destination calls have been
     // heard on local RF.
-    if (!heard_via_tnc_in_past_hour(call)==1        // Haven't heard destination call in previous hour
-            || heard_via_tnc_in_past_hour(from)) {  // Have heard source call in previous hour
+    if (  (   (!object_name && !check_NWS_stations(from)) // Source call not in nws-stations.txt
+            || (object_name && !check_NWS_stations(object_name))) // Object/item not in nws-stations.txt
+        && (!heard_via_tnc_in_past_hour(call)==1        // Haven't heard destination call in previous hour
+            || heard_via_tnc_in_past_hour(from))) {  // Have heard source call in previous hour
 
         if (log_igate && (debug_level & 1024) ) {
             xastir_snprintf(temp,
@@ -931,6 +970,8 @@ end_critical_section(&devices_lock, "igate.c:output_igate_rf" );
                         // unproto_igate path
                         output_my_data(line,x,0,0,1,NULL);
 
+//fprintf(stderr, "Igating->RF: %s\n", line);
+
 begin_critical_section(&devices_lock, "igate.c:output_igate_rf" );
 
                     }
@@ -999,6 +1040,14 @@ void load_NWS_stations(char *file) {
     if (file[0] == '\0')
         return;
 
+    if (NWS_station_data) {
+        free(NWS_station_data);
+        NWS_station_data = NULL;
+    }
+
+    NWS_stations = 0;
+    max_NWS_stations = 0;
+
     f = fopen(file,"r");
     if (f!=NULL) {
         while (!feof(f)) {
@@ -1011,7 +1060,9 @@ void load_NWS_stations(char *file) {
                         // add data
                         // Note:  Size of string variable is 12
                         // bytes, defined in igate.h
-                        (void)sscanf(line,"%11s",NWS_station_data[NWS_stations-1].call);
+                        if (1 != sscanf(line,"%11s",NWS_station_data[NWS_stations-1].call)) {
+                            fprintf(stderr,"load_NWS_stations: sscanf parsing error\n");
+                        }
                         if (debug_level & 1024)
                             fprintf(stderr,"LINE:%s\n",line);
                     }
@@ -1051,6 +1102,9 @@ int check_NWS_stations(char *call) {
     if (call[0] == '\0')
         return(0);
 
+    if (NWS_station_data == NULL)
+        return(0);
+
     if (debug_level & 1024)
         fprintf(stderr,"igate.c::check_NWS_stations %s\n", call);
 
