@@ -1,9 +1,9 @@
 /*
- * $Id: location_gui.c,v 1.11 2004/08/19 19:38:15 we7u Exp $
+ * $Id: location_gui.c,v 1.15 2005/07/31 06:19:12 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2004  The Xastir Group
+ * Copyright (C) 2000-2005  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -134,7 +134,9 @@ void location_view(/*@unused@*/ Widget w, /*@unused@*/ XtPointer clientData, /*@
                             "%s",
                             temp_ptr);
                         if (strcmp(location,name)==0) {
-                            (void)sscanf(pos,"%19s %19s %9s", s_lat, s_long, s_sz);
+                            if (3 != sscanf(pos,"%19s %19s %9s", s_lat, s_long, s_sz)) {
+                                fprintf(stderr,"location_view:sscanf parsing error\n");
+                            }
                             map_pos(convert_lat_s2l(s_lat),convert_lon_s2l(s_long),atol(s_sz));
                             done=1;
                         }
@@ -335,7 +337,7 @@ void location_add(/*@unused@*/ Widget w, XtPointer clientData, /*@unused@*/ XtPo
             else
                 fprintf(stderr,"Couldn't open file: %s\n", get_user_base_dir("config/locations.sys") );
         } else
-            popup_message(langcode("POPEM00022"),langcode("POPEM00023"));
+            popup_message_always(langcode("POPEM00022"),langcode("POPEM00023"));
 
         /* resort the list and put it back up */
         n=1;
@@ -343,7 +345,7 @@ void location_add(/*@unused@*/ Widget w, XtPointer clientData, /*@unused@*/ XtPo
         jump_sort();
         sort_list(get_user_base_dir("data/locations_db.dat"),200,location_list,&n);
     } else
-        popup_message(langcode("POPEM00022"),langcode("POPEM00024")); /* dupe name */
+        popup_message_always(langcode("POPEM00022"),langcode("POPEM00024")); /* dupe name */
 }
 
 
@@ -356,7 +358,7 @@ void location_add(/*@unused@*/ Widget w, XtPointer clientData, /*@unused@*/ XtPo
 void Jump_location(/*@unused@*/ Widget w, /*@unused@*/ XtPointer clientData, /*@unused@*/ XtPointer callData) {
     static Widget  pane,form, button_ok, button_add, button_delete, button_cancel, locdata, location_name;
     int n;
-    Arg al[20];           /* Arg List */
+    Arg al[50];           /* Arg List */
     unsigned int ac = 0;           /* Arg Count */
     Atom delw;
 
