@@ -1,9 +1,9 @@
 /* -*- c-basic-offset: 4; indent-tabs-mode: nil -*-
- * $Id: map_dos.c,v 1.12 2004/08/19 06:38:21 we7u Exp $
+ * $Id: map_dos.c,v 1.16 2005/07/26 19:16:59 tvrusso Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2004  The Xastir Group
+ * Copyright (C) 2000-2005  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -310,11 +310,12 @@ void draw_dos_map(Widget w,
              alert_entry *alert,
              u_char alert_color,
              int destination_pixmap,
-             int draw_filled) {  
+             map_draw_flags *mdf) {  
   FILE *f;
   char file[MAX_FILENAME];
+  char short_filenm[MAX_FILENAME];
   char map_it[MAX_FILENAME];
-  
+
   /* map header info */
   char map_type[5];
   char map_version[5];
@@ -376,7 +377,11 @@ void draw_dos_map(Widget w,
   char symbol_id;
   char symbol_color;
   int embedded_object;
+
+  int draw_filled;
   
+  draw_filled=mdf->draw_filled;
+
   x = 0;
   y = 0;
   color = -1;
@@ -394,6 +399,26 @@ void draw_dos_map(Widget w,
   npoints = 0;
   
   xastir_snprintf(file, sizeof(file), "%s/%s", dir, filenm);
+
+  // Create a shorter filename for display (one that fits the
+  // status line more closely).  Subtract the length of the
+  // "Indexing " and/or "Loading " strings as well.
+  if (strlen(filenm) > (41 - 9)) {
+      int avail = 41 - 11;
+      int new_len = strlen(filenm) - avail;
+
+      xastir_snprintf(short_filenm,
+          sizeof(short_filenm),
+          "..%s",
+          &filenm[new_len]);
+  }
+  else {
+      xastir_snprintf(short_filenm,
+          sizeof(short_filenm),
+          "%s",
+          filenm);
+  }
+
   f = fopen (file, "r");
   if (f != NULL) {
     (void)fread (map_type, 4, 1, f);
@@ -553,15 +578,19 @@ void draw_dos_map(Widget w,
       
       // We're indexing only.  Save the extents in the index.
       index_update_xastir(filenm, // Filename only
-                          bottom_boundary,        // Bottom
-                          top_boundary,           // Top
-                          left_boundary,          // Left
-                          right_boundary);        // Right
+                          bottom_boundary,  // Bottom
+                          top_boundary,     // Top
+                          left_boundary,    // Left
+                          right_boundary,   // Right
+                          1000);            // Default Map Level
       
       (void)fclose (f);
 
       // Update the statusline 
-      xastir_snprintf(map_it, sizeof(map_it), langcode ("BBARSTA039"), filenm);
+      xastir_snprintf(map_it,
+          sizeof(map_it),
+          langcode ("BBARSTA039"),
+          short_filenm);
       statusline(map_it,0);       // Loading/Indexing ...
      
       return; // Done indexing this file
@@ -583,7 +612,10 @@ void draw_dos_map(Widget w,
 
 
       // Update the statusline
-      xastir_snprintf(map_it, sizeof(map_it), langcode ("BBARSTA028"), filenm);
+      xastir_snprintf(map_it,
+          sizeof(map_it),
+          langcode ("BBARSTA028"),
+          short_filenm);
       statusline(map_it,0);       // Loading/Indexing ...
 
       object_behavior = '\0';
