@@ -1,9 +1,9 @@
 /* -*- c-basic-offset: 4; indent-tabs-mode: nil -*-
- * $Id: map_geo.c,v 1.33 2004/08/19 06:38:23 we7u Exp $
+ * $Id: map_geo.c,v 1.56 2005/07/26 19:17:00 tvrusso Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2004  The Xastir Group
+ * Copyright (C) 2000-2005  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -60,6 +60,7 @@
 
 #include "xastir.h"
 #include "maps.h"
+#include "map_cache.h"
 #include "alert.h"
 #include "util.h"
 #include "main.h"
@@ -125,23 +126,14 @@
 #undef XASTIR_PACKAGE_VERSION
 #endif // HAVE_IMAGEMAGICK
 
-#ifdef HAVE_LIBCURL
-#include <curl/curl.h>
-#include <curl/types.h>
-#include <curl/easy.h>
 
-struct FtpFile {
-  char *filename;
-  FILE *stream;
-};
-#endif
 
 extern int npoints;		/* tsk tsk tsk -- globals */
 extern int mag;
 
 void draw_geo_image_map (Widget w, char *dir, char *filenm, 
     alert_entry *alert, u_char alert_color, int destination_pixmap,
-    int draw_filled);
+    map_draw_flags *mdf);
 
 
 
@@ -175,6 +167,22 @@ int check_trans (XColor c, unsigned long c_trans_color) {
 
 
 
+// Regarding MAP CACHING for toporama maps:  Here we are only
+// snagging a .geo file for toporama from findu.com:  We send the
+// parameters off to findu.com, it computes the .geo file, we
+// download it, then we call draw_geo_image_map() with it.  We would
+// have to cache the .geo files from findu, we'd have to modify them
+// to point to our local cached map file (maybe), and we'd of course
+// have to cache that map image as well.  The image filename on
+// findu changes each time we call with the same parameters, so we'd
+// have to give the image file our own name based on the parameters
+// and write that same name into the cached .geo file.  A bit of
+// work, but it _could_ be done.  Actually, we should be able to map
+// between the original URL we use to request the .GEO file and the
+// final image we received.  That way the name would stay the same
+// each time we made the request.  There may be other things we need
+// from the generated .GEO file though.
+//
 // For this particular case we need to snag a remote .geo file and
 // then start the process all over again.  The URL we'll need to use
 // looks something like this:
@@ -191,7 +199,7 @@ void draw_toporama_map (Widget w,
         alert_entry *alert,
         u_char alert_color,
         int destination_pixmap,
-        int draw_filled,
+        map_draw_flags *mdf,
         int toporama_flag) {    // 50 or 250
 
 #ifdef HAVE_IMAGEMAGICK
@@ -200,22 +208,33 @@ void draw_toporama_map (Widget w,
     char map_it[MAX_FILENAME];
     char local_filename[MAX_FILENAME];
     char file[MAX_FILENAME+1];      // Complete path/name of image file
+    char short_filenm[MAX_FILENAME+1];
     FILE *f;                        // Filehandle of image file
     double lat_center = 0;
     double long_center = 0;
     double left, right, top, bottom;
     int my_screen_width, my_screen_height;
     float my_zoom = 1.0;
-#ifdef HAVE_LIBCURL
-    CURL *curl;
-    CURLcode res;
-    char curlerr[CURL_ERROR_SIZE];
-    struct FtpFile ftpfile;
-#else
-#ifdef HAVE_WGET
-    char tempfile[MAX_FILENAME];
-#endif  // HAVE_WGET
-#endif  // HAVE_LIBCURL
+
+
+    // Create a shorter filename for display (one that fits the
+    // status line more closely).  Subtract the length of the
+    // "Indexing " and/or "Loading " strings as well.
+    if (strlen(filenm) > (41 - 9)) {
+        int avail = 41 - 11;
+        int new_len = strlen(filenm) - avail;
+
+        xastir_snprintf(short_filenm,
+            sizeof(short_filenm),
+            "..%s",
+            &filenm[new_len]);
+    }
+    else {
+        xastir_snprintf(short_filenm,
+            sizeof(short_filenm),
+            "%s",
+            filenm);
+    }
 
 
     //fprintf(stderr, "Found TOPORAMA in a .geo file, %dk scale\n", toporama_flag);
@@ -231,10 +250,14 @@ void draw_toporama_map (Widget w,
             64800000l,      // Bottom
             0l,             // Top
             0l,             // Left
-            129600000l);    // Right
+            129600000l,     // Right
+            0);             // Default Map Level
 
         // Update statusline
-        xastir_snprintf(map_it, sizeof(map_it), langcode ("BBARSTA039"), filenm);
+        xastir_snprintf(map_it,
+            sizeof(map_it),
+            langcode ("BBARSTA039"),
+            short_filenm);
         statusline(map_it,0);       // Loading/Indexing ...
 
         return; // Done indexing this file
@@ -332,70 +355,11 @@ void draw_toporama_map (Widget w,
     // remote file, then go back and rework all of the various map
     // routines to use it.
 
-#ifdef HAVE_LIBCURL
-    curl = curl_easy_init();
-
-    if (curl) { 
-
-        /* verbose debug is keen */
-        //curl_easy_setopt(curl, CURLOPT_VERBOSE, TRUE);
-        curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, curlerr);
-
-        /* write function */
-        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, curl_fwrite);
-
-        /* download from fileimg */
-        curl_easy_setopt(curl, CURLOPT_URL, fileimg);
-
-        /* save as local_filename */
-        ftpfile.filename = local_filename;
-        ftpfile.stream = NULL;
-        curl_easy_setopt(curl, CURLOPT_FILE, &ftpfile);    
-
-        res = curl_easy_perform(curl);
-
-        curl_easy_cleanup(curl);
-
-        if (CURLE_OK != res) {
-            fprintf(stderr, "curl told us %d\n", res);
-            fprintf(stderr, "curlerr is %s\n", curlerr);
-        }
-
-        if (ftpfile.stream)
-            fclose(ftpfile.stream);
-
-        // Return if we had trouble
-        if (CURLE_OK != res) {
-            return;
-        }
-
-    } else { 
-        fprintf(stderr,"Couldn't download the toporama .geo file\n");
+    if (fetch_remote_file(fileimg, local_filename)) {
+        // Had trouble getting the file.  Abort.
         return;
     }
 
-
-#else
-#ifdef HAVE_WGET
-    xastir_snprintf(tempfile, sizeof(tempfile),
-        "%s --server-response --timestamping --tries=1 --timeout=30 --output-document=%s \'%s\' 2> /dev/null\n",
-        WGET_PATH,
-        local_filename,
-        fileimg);
-
-    if (debug_level & 16)
-        fprintf(stderr,"%s",tempfile);
-
-//fprintf(stderr,"Getting file\n");
-    if ( system(tempfile) ) {   // Go get the file
-        fprintf(stderr,"Couldn't download the toporama geo file\n");
-        return;
-    }
-#else   // HAVE_WGET
-    fprintf(stderr,"libcurl or 'wget' not installed.  Can't download toporama .geo file\n");
-#endif  // HAVE_WGET
-#endif  // HAVE_LIBCURL
-
     // Set permissions on the file so that any user can overwrite it.
     chmod(local_filename, 0666);
 
@@ -444,7 +408,7 @@ void draw_toporama_map (Widget w,
         alert,
         alert_color,
         destination_pixmap,
-        draw_filled);
+        mdf);
 
 #endif  // HAVE_IMAGEMAGICK
 }
@@ -470,11 +434,12 @@ void draw_geo_image_map (Widget w,
 			 alert_entry *alert,
 			 u_char alert_color,
 			 int destination_pixmap,
-			 int draw_filled) {
+			 map_draw_flags *mdf) {
 #ifdef NO_GRAPHICS
     fprintf(stderr,"XPM and/or ImageMagick support have not been compiled in.");
 #else   // NO_GRAPHICS
     char file[MAX_FILENAME+1];      // Complete path/name of image file
+    char short_filenm[MAX_FILENAME+1];
     FILE *f;                        // Filehandle of image file
     char line[MAX_FILENAME];        // One line from GEO file
     char fileimg[MAX_FILENAME+1];   // Ascii name of image file, read from GEO file
@@ -537,16 +502,7 @@ void draw_geo_image_map (Widget w,
     IndexPacket *index_pack;
     int l;
     XColor my_colors[256];
-#ifdef HAVE_LIBCURL
-    CURL *curl;
-    CURLcode res;
-    char curlerr[CURL_ERROR_SIZE];
-    struct FtpFile ftpfile;
-#else
-#ifdef HAVE_WGET
-    char tempfile[MAX_FILENAME];
-#endif  // HAVE_WGET
-#endif  // HAVE_LIBCURL
+    time_t query_start_time, query_end_time;
     char gamma[16];
     struct {
         float r_gamma;
@@ -572,17 +528,19 @@ void draw_geo_image_map (Widget w,
     XImage *xi;                 // Temp XImage used for reading in current image
 #endif // HAVE_IMAGEMAGICK
 
-    int terraserver_flag = 0;   // U.S. satellite images via terraserver
-    int toposerver_flag = 0;    // U.S. topo's via terraserver
+    int terraserver_flag = 0;   // U.S. satellite images/topo/reflectivity/urban
+                                // areas via terraserver
     int tigerserver_flag = 0;   // U.S. Street maps via census.gov
     int toporama_flag = 0;      // Canadian topo's from mm.aprs.net (originally from Toporama)
+    int WMSserver_flag = 0;     // WMS server
     char map_it[MAX_FILENAME];
     int geo_image_width = 0;    // Image width  from GEO file
     int geo_image_height = 0;   // Image height from GEO file
     char geo_datum[8+1];        // WGS-84 etc.
     char geo_projection[256+1];   // TM, UTM, GK, LATLON etc.
     int map_proj=0;
-    int map_refresh_interval_temp;
+    int map_refresh_interval_temp = 0;
+    int nocache = 0;            // Don't cache the file if non-zero
 #ifdef FUZZYRASTER
     int rasterfuzz = 3;    // ratio to skip 
 #endif //FUZZYRASTER
@@ -595,10 +553,34 @@ void draw_geo_image_map (Widget w,
 #ifdef TIMING_DEBUG
     time_mark(1);
 #endif  // TIMING_DEBUG
+#ifdef USE_MAP_CACHE 
+    int map_cache_return;
+#endif  // USE_MAP_CACHE
+
 
 
     xastir_snprintf(file, sizeof(file), "%s/%s", dir, filenm);
 
+    // Create a shorter filename for display (one that fits the
+    // status line more closely).  Subtract the length of the
+    // "Indexing " and/or "Loading " strings as well.
+    if (strlen(filenm) > (41 - 9)) {
+        int avail = 41 - 11;
+        int new_len = strlen(filenm) - avail;
+
+        xastir_snprintf(short_filenm,
+            sizeof(short_filenm),
+            "..%s",
+            &filenm[new_len]);
+    }
+    else {
+        xastir_snprintf(short_filenm,
+            sizeof(short_filenm),
+            "%s",
+            filenm);
+    }
+
+
     // Read the .geo file to find out map filename and tiepoint info
     n_tp = 0;
     geo_datum[0]      = '\0';
@@ -608,7 +590,9 @@ void draw_geo_image_map (Widget w,
         while (!feof (f)) {
             (void)get_line (f, line, MAX_FILENAME);
             if (strncasecmp (line, "FILENAME", 8) == 0) {
-                (void)sscanf (line + 9, "%s", fileimg);
+                if (1 != sscanf (line + 9, "%s", fileimg)) {
+                    fprintf(stderr,"draw_geo_image_map:sscanf parsing error\n");
+                }
                 if (fileimg[0] != '/' ) { // not absolute path
                     // make it relative to the .geo file
                     char temp[MAX_FILENAME];
@@ -625,11 +609,19 @@ void draw_geo_image_map (Widget w,
                 }
             }
             if (strncasecmp (line, "URL", 3) == 0)
-                (void)sscanf (line + 4, "%s", fileimg);
+                if (1 != sscanf (line + 4, "%s", fileimg)) {
+                    fprintf(stderr,"draw_geo_image_map:sscanf parsing error\n");
+                }
 
             if (n_tp < 2) {     // Only take the first two tiepoints
                 if (strncasecmp (line, "TIEPOINT", 8) == 0) {
-                    (void)sscanf (line + 9, "%d %d %f %f",&tp[n_tp].img_x,&tp[n_tp].img_y,&temp_long,&temp_lat);
+                    if (4 != sscanf (line + 9, "%d %d %f %f",
+                            &tp[n_tp].img_x,
+                            &tp[n_tp].img_y,
+                            &temp_long,
+                            &temp_lat)) {
+                        fprintf(stderr,"draw_geo_image_map:sscanf parsing error\n");
+                    }
                     // Convert tiepoints from lat/lon to Xastir coordinates
                     tp[n_tp].x_long = 64800000l + (360000.0 * temp_long);
                     tp[n_tp].y_lat  = 32400000l + (360000.0 * (-temp_lat));
@@ -638,30 +630,46 @@ void draw_geo_image_map (Widget w,
             }
 
             if (strncasecmp (line, "IMAGESIZE", 9) == 0)
-                (void)sscanf (line + 10, "%d %d",&geo_image_width,&geo_image_height);
+                if (2 != sscanf (line + 10, "%d %d",&geo_image_width,&geo_image_height)) {
+                    fprintf(stderr,"draw_geo_image_map:sscanf parsing error\n");
+                }
 
             if (strncasecmp (line, "DATUM", 5) == 0)
-                (void)sscanf (line + 6, "%8s",geo_datum);
+                if (1 != sscanf (line + 6, "%8s",geo_datum)) {
+                    fprintf(stderr,"draw_geo_image_map:sscanf parsing error\n");
+                }
 
             if (strncasecmp (line, "PROJECTION", 10) == 0)
-                (void)sscanf (line + 11, "%256s",geo_projection); // ignores leading and trailing space (nice!)
+                // Ignores leading and trailing space (nice!)
+                if (1 != sscanf (line + 11, "%256s",geo_projection)) {
+                    fprintf(stderr,"draw_geo_image_map:sscanf parsing error\n");
+                }
 
-            if (strncasecmp (line, "TERRASERVER", 11) == 0)
-                terraserver_flag = 1;
+            if (strncasecmp (line, "TERRASERVER-URBAN", 17) == 0)
+                terraserver_flag = 4;
 
-            if (strncasecmp (line, "TIGERMAP", 8) == 0)
-                tigerserver_flag = 1;
+            if (strncasecmp (line, "TERRASERVER-REFLECTIVITY", 24) == 0)
+                terraserver_flag = 3;
 
-            if (strncasecmp (line, "TOPOSERVER", 10) == 0) {
+            if (strncasecmp (line, "TERRASERVER-TOPO", 16) == 0) {
                 // Set to max brightness as it looks weird when the
                 // intensity variable comes into play.
 #ifdef HAVE_IMAGEMAGICK
 // This one causes problems now.  Not sure why.
 //                xastir_snprintf(imagemagick_options.modulate,32,"100 100 100");
 #endif  // HAVE_IMAGEMAGICK
-                toposerver_flag = 1;
+                terraserver_flag = 2;
             }
 
+            if (strncasecmp (line, "TERRASERVER-SATELLITE", 21) == 0)
+                terraserver_flag = 1;
+
+            if (strncasecmp (line, "TIGERMAP", 8) == 0)
+                tigerserver_flag = 1;
+
+            if (strncasecmp (line, "WMSSERVER", 9) == 0)
+                WMSserver_flag = 1;
+
             // Check for Canadian topo map request
             if (strncasecmp (line, "TOPORAMA-50k", 12) == 0) {
                 toporama_flag = 50;
@@ -678,14 +686,19 @@ void draw_geo_image_map (Widget w,
                     && (destination_pixmap != INDEX_NO_TIMESTAMPS) ) {
 
                 if (strncasecmp (line, "REFRESH", 7) == 0) {
-                    (void)sscanf (line + 8, "%d", &map_refresh_interval_temp);
+                    if (1 != sscanf (line + 8, "%d", &map_refresh_interval_temp)) {
+                        fprintf(stderr,"draw_geo_image_map:sscanf parsing error\n");
+                    }
                     if ( map_refresh_interval_temp > 0 && 
                          ( map_refresh_interval == 0 || 
                            map_refresh_interval_temp < map_refresh_interval) ) {
                         map_refresh_interval = (time_t) map_refresh_interval_temp;
                         map_refresh_time = sec_now() + map_refresh_interval;
-                        fprintf(stderr, "Map Refresh set to %d.\n", (int) map_refresh_interval);
+
+                        if (debug_level & 512)
+                            fprintf(stderr, "Map Refresh set to %d.\n", (int) map_refresh_interval);
                     }
+                    nocache = map_refresh_interval_temp;
                 }
             }
 
@@ -693,7 +706,9 @@ void draw_geo_image_map (Widget w,
                 // need to make this read a list of colors to zap
                 // out.  Use 32-bit unsigned values, so we can
                 // handle 32-bit color displays.
-                (void)sscanf (line + 12, "%lx", &trans_color);
+                if (1 != sscanf (line + 12, "%lx", &trans_color)) {
+                    fprintf(stderr,"draw_geo_image_map:sscanf parsing error\n");
+                }
 
                 // Mask it with the color depth so that we don't
                 // check too many bits when looking for the
@@ -714,8 +729,13 @@ void draw_geo_image_map (Widget w,
 //fprintf(stderr,"New Transparent: %lx\n",trans_color);
             }
             if (strncasecmp(line, "CROP", 4) == 0) { 
-                (void)sscanf (line + 5, "%d %d %d %d", 
-                              &crop_x1,&crop_y1,&crop_x2,&crop_y2 ); 
+                if (4 != sscanf (line + 5, "%d %d %d %d", 
+                        &crop_x1,
+                        &crop_y1,
+                        &crop_x2,
+                        &crop_y2 )) {
+                    fprintf(stderr,"draw_geo_image_map:sscanf parsing error\n");
+                }
                 if (crop_x1 < 0 ) crop_x1 = 0;
                 if (crop_y1 < 0 ) crop_y1 = 0;
                 if (crop_x2 < 0 ) crop_x2 = 0;
@@ -732,10 +752,16 @@ void draw_geo_image_map (Widget w,
                                                         &imagemagick_options.r_gamma,
                                                         &imagemagick_options.g_gamma,
                                                         &imagemagick_options.b_gamma);
-            if (strncasecmp(line, "CONTRAST", 8) == 0)
-                (void)sscanf(line + 9, "%d", &imagemagick_options.contrast);
-            if (strncasecmp(line, "NEGATE", 6) == 0)
-                (void)sscanf(line + 7, "%d", &imagemagick_options.negate);
+            if (strncasecmp(line, "CONTRAST", 8) == 0) {
+                if (1 != sscanf(line + 9, "%d", &imagemagick_options.contrast)) {
+                    fprintf(stderr,"draw_geo_image_map:sscanf parsing error\n");
+                }
+            }
+            if (strncasecmp(line, "NEGATE", 6) == 0) {
+                if (1 != sscanf(line + 7, "%d", &imagemagick_options.negate)) {
+                    fprintf(stderr,"draw_geo_image_map:sscanf parsing error\n");
+                }
+            }
             if (strncasecmp(line, "EQUALIZE", 8) == 0)
                 imagemagick_options.equalize = 1;
             if (strncasecmp(line, "NORMALIZE", 9) == 0)
@@ -770,7 +796,35 @@ void draw_geo_image_map (Widget w,
     if (tigerserver_flag) {
 
 #ifdef HAVE_IMAGEMAGICK
-        draw_tiger_map(w, filenm, destination_pixmap);
+
+        // We need to send the "nocache" parameter to this function
+        // for those instances when the tigermap received is bad.
+        // Later the GUI can implement a method for refreshing the
+        // latest map and replacing the bad map in the cache.
+        //
+        draw_tiger_map(w, filenm, destination_pixmap, nocache);
+
+#endif  // HAVE_IMAGEMAGICK
+
+        return;
+    }
+
+
+    // Check whether a WMS server has been selected.  If so, run off
+    // to another routine to service this request.
+    //
+    if (WMSserver_flag) {
+
+#ifdef HAVE_IMAGEMAGICK
+        // Pass the URL in "fileimg"
+        draw_WMS_map(w,
+            filenm,
+            destination_pixmap,
+            fileimg,
+            do_check_trans,
+            trans_color,
+            nocache); // Don't use cached version if non-zero
+
 #endif  // HAVE_IMAGEMAGICK
 
         return;
@@ -790,7 +844,7 @@ void draw_geo_image_map (Widget w,
             alert,
             alert_color,
             destination_pixmap,
-            draw_filled,
+            mdf,
             toporama_flag);
 #endif  // HAVE_IMAGEMAGICK
 
@@ -829,52 +883,136 @@ void draw_geo_image_map (Widget w,
         map_proj = 0;           // Lat/Lon, default
 
 #ifdef HAVE_IMAGEMAGICK
-    if (terraserver_flag || toposerver_flag) {
+    if (terraserver_flag) {
 //http://terraservice.net/download.ashx?t=1&s=10&x=2742&y=26372&z=10&w=820&h=480
         if (scale_y <= 4) {
-                t_zoom  = 10; // 1m
+                t_zoom  = 10; // 1m/pixel
                 t_scale = 200;
         }
         else if (scale_y <= 8) {
-             t_zoom  = 11; // 2m
+             t_zoom  = 11; // 2m/pixel
              t_scale = 400;
         }
         else if (scale_y <= 16) {
-            t_zoom  = 12; // 4m
+            t_zoom  = 12; // 4m/pixel
             t_scale = 800;
         }
         else if (scale_y <= 32) {
-            t_zoom  = 13; // 8m
+            t_zoom  = 13; // 8m/pixel
             t_scale = 1600;
         }
         else if (scale_y <= 64) {
-            t_zoom  = 14; // 16m
+            t_zoom  = 14; // 16m/pixel
             t_scale = 3200;
         }
         else if (scale_y <= 128) {
-            t_zoom  = 15; // 32m
+            t_zoom  = 15; // 32m/pixel
             t_scale = 6400;
         }
         else {
-            t_zoom  = 16; // 64m
+            t_zoom  = 16; // 64m/pixel
             t_scale = 12800;
         }
 
         top  = -((y_lat_offset - 32400000l) / 360000.0);
         left =  (x_long_offset - 64800000l) / 360000.0;
-        ll_to_utm_ups(gDatum[D_NAD_83_CONUS].ellipsoid, top, left, &top_n, &left_e, zstr, sizeof(zstr) );
-        sscanf(zstr, "%d", &z);
+        ll_to_utm_ups(gDatum[D_NAD_83_CONUS].ellipsoid,
+            top,
+            left,
+            &top_n,
+            &left_e,
+            zstr,
+            sizeof(zstr) );
+        if (1 != sscanf(zstr, "%d", &z)) {
+            fprintf(stderr,"draw_geo_image_map:sscanf parsing error\n");
+        }
 
         bottom = -(((y_lat_offset + (screen_height * scale_y)) - 32400000l) / 360000.0);
         right  =   ((x_long_offset + (screen_width * scale_x)) - 64800000l) / 360000.0;
-        ll_to_utm_ups(gDatum[D_NAD_83_CONUS].ellipsoid, bottom, right, &bottom_n, &right_e, zstr, sizeof(zstr) );
+        ll_to_utm_ups(gDatum[D_NAD_83_CONUS].ellipsoid,
+            bottom,
+            right,
+            &bottom_n,
+            &right_e,
+            zstr,
+            sizeof(zstr) );
 
         map_top_n  = (int)((top_n  / t_scale) + 1) * t_scale;
         map_left_e = (int)((left_e / t_scale) + 0) * t_scale;
-        utm_ups_to_ll(gDatum[D_NAD_83_CONUS].ellipsoid, map_top_n, map_left_e, zstr, &top, &left);
+        utm_ups_to_ll(gDatum[D_NAD_83_CONUS].ellipsoid,
+            map_top_n,
+            map_left_e,
+            zstr,
+            &top,
+            &left);
+
+
+
+// Below here things can get messed up.  We can end up with very
+// large and/or negative values for geo_image_width and/or
+// geo_image_height.  Usually happens around UTM zone boundaries.
+//
+// Terraserver uses UTM coordinates for specifying the maps instead
+// of lat/long.  Note that we're also not supposed to cross UTM
+// zones in our requests.
+//
+// t = 1 - 4, theme.  1=DOQ (aerial photo)
+//                    2=DRG (topo)
+//                    3=shaded relief
+//                    4=Color photos/Urban areas
+// s = 10 - 16, scale.  10=1 meter/pixel.  11=2 meters/pixel.
+// x = UTM easting, center of image
+// y = UTM northing, center of image
+// z = 1 - 60, UTM zone, center of image
+// w = 50 - 2000, width in pixels
+// h = 50 - 2000, height in pixels
+// logo = 0/1, USGS logo in image if 1
+//
+
+        // This number gets messed up if we cross zones.  UTM lines
+        // are slanted, so we _can_ cross zones vertically!
+        geo_image_height = abs(map_top_n - bottom_n) * 200 / t_scale;
+
+        // This number gets messed up if we cross zones
+        geo_image_width  = abs(right_e - map_left_e) * 200 / t_scale;
+
+ 
+//fprintf(stderr,"\ngeo_image_height:%d\tmap_top_n:%0.1f\tbottom_n:%0.1f\tt_scale:%d\n",
+//geo_image_height,
+//map_top_n,
+//bottom_n,
+//t_scale);
+// map_top_n is the one that goes whacko, throwing off the height.
+// Check whether this is because we're crossing a UTM zone.  We
+// _can_ cross zones vertically because the UTM lines are slanted.
+
+//fprintf(stderr,"geo_image_width:%d\tright_e:%0.1f\tmap_left_e:%0.1f\tt_scale:%d\n",
+//geo_image_width,
+//right_e,
+//map_left_e,
+//t_scale);
+// right_e is the one that goes whacko, throwing off the width.
+// Check whether this is because we're crossing a UTM zone.
+
+
+        if (geo_image_height < 50)
+            geo_image_height = 50;
+
+        if (geo_image_width < 50)
+            geo_image_width = 50;
+
+        if (geo_image_height > 2000)
+            geo_image_height = geo_image_width;
+ 
+        if (geo_image_width > 2000)
+            geo_image_width = geo_image_height;
+
+        if (geo_image_height > 2000)
+            geo_image_height = geo_image_width;
+         if (geo_image_width > 2000)
+            geo_image_width = geo_image_height;
+
 
-        geo_image_height = (map_top_n - bottom_n) * 200 / t_scale;
-        geo_image_width  = (right_e - map_left_e) * 200 / t_scale;
         map_width  = right - left;
         map_height = top - bottom;
 
@@ -888,21 +1026,27 @@ void draw_geo_image_map (Widget w,
         tp[1].x_long = 64800000l + (360000.0 * right);
         tp[1].y_lat  = 32400000l + (360000.0 * (-bottom));
 
+
         url_n = (int)(top_n  / t_scale); // The request URL does not use the
         url_e = (int)(left_e / t_scale); // N/E of the map corner
 
+
         xastir_snprintf(fileimg, sizeof(fileimg),
             "http://terraservice.net/download.ashx?t=%d\046s=%d\046x=%d\046y=%d\046z=%d\046w=%d\046h=%d",
 //            "http://terraserver-usa.net/download.ashx?t=%d\046s=%d\046x=%d\046y=%d\046z=%d\046w=%d\046h=%d",
-            (toposerver_flag) ? 2 : 1,
+            terraserver_flag,   // 1, 2, 3, or 4
             t_zoom,
-            url_e,
-            url_n,
+            url_e,  // easting, center of map
+            url_n,  // northing, center of map
             z,
             geo_image_width,
             geo_image_height);
 //http://terraservice.net/download.ashx?t=1&s=11&x=1384&y=13274&z=10&w=1215&h=560
 //fprintf(stderr,"%s\n",fileimg);
+
+        if (debug_level & 16) {
+            fprintf(stderr,"URL: %s\n", fileimg);
+        }
     }
 #endif // HAVE_IMAGEMAGICK
 
@@ -1062,25 +1206,30 @@ fprintf(stderr,"1 ");
             || (destination_pixmap == INDEX_NO_TIMESTAMPS) ) {
 
         // We're indexing only.  Save the extents in the index.
-        if (terraserver_flag || toposerver_flag) {
+        if (terraserver_flag) {
             // Force the extents to the edges of the earth for the
             // index file.
             index_update_xastir(filenm, // Filename only
                 64800000l,      // Bottom
                 0l,             // Top
                 0l,             // Left
-                129600000l);    // Right
+                129600000l,     // Right
+                0);             // Default Map Level
         }
         else {
             index_update_xastir(filenm, // Filename only
                 tp[1].y_lat,    // Bottom
                 tp[0].y_lat,    // Top
                 tp[0].x_long,   // Left
-                tp[1].x_long);  // Right
+                tp[1].x_long,   // Right
+                0);             // Default Map Level
         }
 
         // Update statusline
-        xastir_snprintf(map_it, sizeof(map_it), langcode ("BBARSTA039"), filenm);
+        xastir_snprintf(map_it,
+            sizeof(map_it),
+            langcode ("BBARSTA039"),
+            short_filenm);
         statusline(map_it,0);       // Loading/Indexing ...
 
         return; // Done indexing this file
@@ -1120,7 +1269,10 @@ fprintf(stderr,"1 ");
     }
 
     // Update statusline
-    xastir_snprintf(map_it, sizeof(map_it), langcode ("BBARSTA028"), filenm);
+    xastir_snprintf(map_it,
+        sizeof(map_it),
+        langcode ("BBARSTA028"),
+        short_filenm);
     statusline(map_it,0);       // Loading/Indexing ...
 
 #ifndef NO_XPM
@@ -1144,93 +1296,104 @@ fprintf(stderr,"1 ");
     // Check to see if we have to use "wget" to go get an internet map
     if ( (strncasecmp ("http", fileimg, 4) == 0)
             || (strncasecmp ("ftp", fileimg, 3) == 0)
-            || (terraserver_flag)
-            || (toposerver_flag) ) {
+            || (terraserver_flag) ) {
 #ifdef HAVE_IMAGEMAGICK
         char *ext;
 
         if (debug_level & 16)
             fprintf(stderr,"ftp or http file: %s\n", fileimg);
 
-        if (terraserver_flag || toposerver_flag)
+        if (terraserver_flag)
             ext = "jpg";
         else
             ext = get_map_ext(fileimg); // Use extension to determine image type
 
+        if (debug_level & 512) {
+            query_start_time=time(&query_start_time);
+        }
+
+#ifdef USE_MAP_CACHE
+
+        if (nocache || map_cache_fetch_disable) {
+
+            // Delete old copy from the cache
+            if (map_cache_fetch_disable && fileimg[0] != '\0') {
+                if (map_cache_del(fileimg)) {
+                    if (debug_level & 512) {
+                        fprintf(stderr,"Couldn't delete old map from cache\n");
+                    }
+                }
+            }
+
+            // Simulate a cache miss
+            map_cache_return = 1;
+        }
+        else {
+            // Look for the file in the cache
+            map_cache_return = map_cache_get(fileimg,local_filename);
+        }
+
+        if (debug_level & 512) {
+            fprintf(stderr,"map_cache_return: %d\n", map_cache_return);
+        }
+
+        // If nocache is non-zero, we're supposed to refresh the map
+        // at intervals.  Don't use a cached version of the map in
+        // that case.
+        //
+        if (nocache || map_cache_return != 0 ) {
+
+            // Caching not requested or cached file not found.  We
+            // must snag the remote file via libcurl or wget.
+
+            if (nocache) {
+                xastir_snprintf(local_filename,
+                    sizeof(local_filename),
+                    "%s/map.%s",
+                    get_user_base_dir("tmp"),ext);
+            }
+            else {
+                xastir_snprintf(local_filename,
+                    sizeof(local_filename),
+                    "%s/map_%s.%s",
+                    get_user_base_dir("map_cache"),
+                    map_cache_fileid(),
+                    ext);
+            }
+
+#else   // USE_MAP_CACHE
+
         xastir_snprintf(local_filename,
             sizeof(local_filename),
             "%s/map.%s",
             get_user_base_dir("tmp"),ext);
 
+#endif  // USE_MAP_CACHE
+
+
         // Erase any previously existing local file by the same
         // name.  This avoids the problem of having an old map image
         // here and the code trying to display it when the download
         // fails.
         unlink( local_filename );
 
-#ifdef HAVE_LIBCURL
-        curl = curl_easy_init();
-
-        if (curl) { 
-
-            /* verbose debug is keen */
-          //            curl_easy_setopt(curl, CURLOPT_VERBOSE, TRUE);
-            curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, curlerr);
-
-            /* write function */
-            curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, curl_fwrite);
-
-            /* download from fileimg */
-            curl_easy_setopt(curl, CURLOPT_URL, fileimg);
-
-            /* save as local_filename */
-            ftpfile.filename = local_filename;
-            ftpfile.stream = NULL;
-            curl_easy_setopt(curl, CURLOPT_FILE, &ftpfile);    
-
-            res = curl_easy_perform(curl);
-
-            curl_easy_cleanup(curl);
-
-            if (CURLE_OK != res) {
-                fprintf(stderr, "curl told us %d\n", res);
-                fprintf(stderr, "curlerr is %s\n", curlerr);
-            }
-
-            if (ftpfile.stream)
-                fclose(ftpfile.stream);
-
-            // Return if we had trouble
-            if (CURLE_OK != res) {
-                return;
-            }
-
-        } else { 
-            fprintf(stderr,"Couldn't download the geo or Terraserver image\n");
+        if (fetch_remote_file(fileimg, local_filename)) {
+            // Had trouble getting the file.  Abort.
             return;
         }
+    
+#ifdef USE_MAP_CACHE
 
-
-#else
-#ifdef HAVE_WGET
-        xastir_snprintf(tempfile, sizeof(tempfile),
-                "%s --server-response --timestamping --tries=1 --timeout=30 --output-document=%s \'%s\' 2> /dev/null\n",
-                WGET_PATH,
-                local_filename,
-                fileimg);
-
-        if (debug_level & 16)
-            fprintf(stderr,"%s",tempfile);
-
-//fprintf(stderr,"Getting file\n");
-        if ( system(tempfile) ) {   // Go get the file
-            fprintf(stderr,"Couldn't download the geo or Terraserver image\n");
-            return;
+        // Cache the map only if map_refresh_interval_temp is zero
+        if (!map_refresh_interval_temp) {
+            map_cache_put(fileimg,local_filename);
         }
-#else   // HAVE_WGET
-        fprintf(stderr,"libcurl or 'wget' not installed.  Can't download image\n");
-#endif  // HAVE_WGET
-#endif  // HAVE_LIBCURL
+
+        } // end if is cached
+#endif // MAP_CACHE
+
+        (debug_level & 512) && fprintf (stderr, "Fetch or query took %d seconds\n", 
+                            (int) (time(&query_end_time) - query_start_time)); 
 
         // Set permissions on the file so that any user can overwrite it.
         chmod(local_filename, 0666);
@@ -1750,6 +1913,18 @@ fprintf(stderr,"2 ");
         return;
     }
 
+    // Check whether file has .xpm or .xpm.Z or .xpm.gz at the end.
+    // If not, don't use the XpmReadFileToImage call below.
+
+    if (       !strstr(filenm,"xpm")
+            && !strstr(filenm,"XPM")
+            && !strstr(filenm,"Xpm") ) {
+        fprintf(stderr,
+            "Error: File format not supported by XPM library: %s\n",
+            filenm);
+        return;
+    }
+
     /*  XpmReadFileToImage is the call we wish to avoid if at all
      *  possible.  On large images this can take quite a while.  We
      *  check above to see whether the image is inside our viewport,
@@ -1761,9 +1936,16 @@ fprintf(stderr,"2 ");
             XDestroyImage (xi);
         return;
     }
+
     if (debug_level & 16) {
         fprintf(stderr,"Image size %d %d\n", (int)atb.width, (int)atb.height);
-        fprintf(stderr,"XX: %ld YY:%ld Sx %f %d Sy %f %d\n", map_c_L, map_c_T, map_c_dx,(int) (map_c_dx / scale_x), map_c_dy, (int) (map_c_dy / scale_y));
+        fprintf(stderr,"XX: %ld YY:%ld Sx %f %d Sy %f %d\n",
+            map_c_L,
+            map_c_T,
+            map_c_dx,
+            (int) (map_c_dx / scale_x),
+            map_c_dy,
+            (int) (map_c_dy / scale_y));
     }
 
     HandlePendingEvents(app_context);
@@ -1952,8 +2134,9 @@ fprintf(stderr,"2 ");
                         if (image->storage_class == PseudoClass) {
                             if ( do_check_trans && 
                                     check_trans(my_colors[index_pack[l]],trans_color) ) {
-                                trans_skip = 1;
-                            } else {
+                                trans_skip = 1; // skip it
+                            }
+                            else {
                                 XSetForeground(XtDisplay(w), gc, my_colors[index_pack[l]].pixel);
                                 trans_skip = 0; // draw it
                             }
@@ -1964,8 +2147,9 @@ fprintf(stderr,"2 ");
                                             &my_colors[0].pixel);
                             if ( do_check_trans && 
                                     check_trans(my_colors[0],trans_color) ) {
-                                trans_skip = 1;
-                            } else {
+                                trans_skip = 1; // skip it
+                            }
+                            else {
                                 XSetForeground(XtDisplay(w), gc, my_colors[0].pixel);
                                 trans_skip = 0; // draw it
                             }
@@ -1975,6 +2159,7 @@ fprintf(stderr,"2 ");
 #endif  // HAVE_IMAGEMAGICK
 
 
+                        // Skip drawing if a transparent pixel
 #ifdef HAVE_IMAGEMAGICK
                         if ( pixel_pack[l].opacity == 0 && !trans_skip  ) // skip transparent
 #else   // HAVE_IMAGEMAGICK
