@@ -1,9 +1,9 @@
 /* -*- c-basic-offset: 4; indent-tabs-mode: nil -*-
- * $Id: map_pdb.c,v 1.7 2004/01/26 16:18:23 we7u Exp $
+ * $Id: map_pdb.c,v 1.11 2005/07/26 19:17:01 tvrusso Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2004  The Xastir Group
+ * Copyright (C) 2000-2005  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -92,7 +92,7 @@ void draw_palm_image_map(Widget w,
 			 alert_entry *alert, 
 			 u_char alert_color,
 			 int destination_pixmap, 
-			 int draw_filled) {
+			 map_draw_flags *mdf) {
 
 // Do NOT change any of these structs.  They have to match the
 // structs  that the palm maps were made with.
@@ -173,6 +173,7 @@ void draw_palm_image_map(Widget w,
 
     FILE *fn;
     char filename[MAX_FILENAME];
+    char short_filenm[MAX_FILENAME];
     int records, record_count, count;
     int scale;
     long map_left, map_right, map_top, map_bottom, max_x, max_y;
@@ -180,10 +181,31 @@ void draw_palm_image_map(Widget w,
     long line_x, line_y;
     int vector;
     char status_text[MAX_FILENAME];
+    int draw_filled;
 
+    draw_filled = mdf->draw_filled;
 
     xastir_snprintf(filename, sizeof(filename), "%s/%s", dir, filenm);
 
+    // Create a shorter filename for display (one that fits the
+    // status line more closely).  Subtract the length of the
+    // "Indexing " and/or "Loading " strings as well.
+    if (strlen(filenm) > (41 - 9)) {
+        int avail = 41 - 11;
+        int new_len = strlen(filenm) - avail;
+
+        xastir_snprintf(short_filenm,
+            sizeof(short_filenm),
+            "..%s",
+            &filenm[new_len]);
+    }
+    else {
+        xastir_snprintf(short_filenm,
+            sizeof(short_filenm),
+            "%s",
+            filenm);
+    }
+
     if ((fn = fopen(filename, "r")) != NULL) {
 
         if (debug_level & 16)
@@ -255,12 +277,16 @@ void draw_palm_image_map(Widget w,
                 map_bottom, // Bottom
                 map_top,    // Top
                 map_left,   // Left
-                map_right); // Right
+                map_right,  // Right
+                1000);      // Default Map Level
 
             fclose(fn);
 
             // Update the statusline for this map name.
-            xastir_snprintf(status_text, sizeof(status_text), langcode ("BBARSTA039"), filenm);
+            xastir_snprintf(status_text,
+                sizeof(status_text),
+                langcode ("BBARSTA039"),
+                short_filenm);
             statusline(status_text,0);       // Loading/Indexing ...
 
             return; // Done indexing this file
@@ -271,7 +297,10 @@ void draw_palm_image_map(Widget w,
 
 
             // Update the statusline for this map name
-            xastir_snprintf(status_text, sizeof(status_text), langcode ("BBARSTA028"), filenm);
+            xastir_snprintf(status_text,
+                sizeof(status_text),
+                langcode ("BBARSTA028"),
+                short_filenm);
             statusline(status_text,0);       // Loading/Indexing ...
 
 
