@@ -1,9 +1,9 @@
 /* -*- c-basic-offset: 4; indent-tabs-mode: nil -*-
- * $Id: map_tif.c,v 1.10 2004/07/30 04:02:42 we7u Exp $
+ * $Id: map_tif.c,v 1.24 2005/07/28 20:41:19 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2004  The Xastir Group
+ * Copyright (C) 2000-2005  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,6 +23,7 @@
  */
 
 
+
 #include "config.h"
 #include "snprintf.h"
 
@@ -172,6 +173,8 @@ void get_alt_fgd_path2(char *fullpath, int fullpath_length) {
 
 
 
+
+
 /***********************************************************
  * read_fgd_file()
  *
@@ -236,7 +239,9 @@ int read_fgd_file ( char* tif_filename,
                 if ( ( (ptr = strstr(line, "WEST BOUNDING COORDINATE:") ) != NULL)
                         || ( (ptr = strstr(line, "West_Bounding_Coordinate:") ) != NULL) )
                 {
-                    sscanf (ptr + 25, " %f", f_west_bounding);
+                    if (1 != sscanf (ptr + 25, " %f", f_west_bounding)) {
+                        fprintf(stderr,"read_fgd_file:sscanf parsing error\n");
+                    }
                     if (debug_level & 512)
                         fprintf(stderr,"West Bounding:  %f\n",*f_west_bounding);
                     num_coordinates++;
@@ -248,7 +253,9 @@ int read_fgd_file ( char* tif_filename,
                 if ( ( (ptr = strstr(line, "EAST BOUNDING COORDINATE:") ) != NULL)
                         || ( (ptr = strstr(line, "East_Bounding_Coordinate:") ) != NULL) )
                 {
-                    sscanf (ptr + 25, " %f", f_east_bounding);
+                    if (1 != sscanf (ptr + 25, " %f", f_east_bounding)) {
+                        fprintf(stderr,"read_fgd_file:sscanf parsing error\n");
+                    }
                     if (debug_level & 512)
                         fprintf(stderr,"East Bounding:  %f\n",*f_east_bounding);
                     num_coordinates++;
@@ -260,7 +267,9 @@ int read_fgd_file ( char* tif_filename,
                 if ( ( (ptr = strstr(line, "NORTH BOUNDING COORDINATE:") ) != NULL)
                         || ( (ptr = strstr(line, "North_Bounding_Coordinate:") ) != NULL) )
                 {
-                    sscanf (ptr + 26, " %f", f_north_bounding);
+                    if (1 != sscanf (ptr + 26, " %f", f_north_bounding)) {
+                        fprintf(stderr,"read_fgd_file:sscanf parsing error\n");
+                    }
                     if (debug_level & 512)
                         fprintf(stderr,"North Bounding: %f\n",*f_north_bounding);
                     num_coordinates++;
@@ -272,7 +281,9 @@ int read_fgd_file ( char* tif_filename,
                 if ( ( (ptr = strstr(line, "SOUTH BOUNDING COORDINATE:") ) != NULL)
                         || ( (ptr = strstr(line, "South_Bounding_Coordinate:") ) != NULL) )
                 {
-                    sscanf (ptr + 26, " %f", f_south_bounding);
+                    if (1 != sscanf (ptr + 26, " %f", f_south_bounding)) {
+                        fprintf(stderr,"read_fgd_file:sscanf parsing error\n");
+                    }
                     if (debug_level & 512)
                         fprintf(stderr,"South Bounding: %f\n",*f_south_bounding);
                     num_coordinates++;
@@ -316,6 +327,9 @@ int read_fgd_file ( char* tif_filename,
 }
 
 
+
+
+
 // Check the "GTIFProj4*" functions to see if we can use
 // datum.c instead for any of them.  These calls are all in
 // draw_geotiff_image_map().  It looks like they deal with
@@ -357,8 +371,9 @@ void draw_geotiff_image_map (Widget w,
 			     alert_entry *alert,
 			     u_char alert_color,
 			     int destination_pixmap,
-			     int draw_filled) {
+			     map_draw_flags *mdf) {
     char file[MAX_FILENAME];    /* Complete path/name of image file */
+    char short_filenm[MAX_FILENAME];
     TIFF *tif = (TIFF *) 0;     /* Filehandle for tiff image file */
     GTIF *gtif = (GTIF *) 0;    /* GeoKey-level descriptor */
     /* enum { VERSION = 0, MAJOR, MINOR }; */
@@ -489,8 +504,12 @@ void draw_geotiff_image_map (Widget w,
     register float total_avg_y_increment;
     unsigned long view_left_minus_pixel_width;
     unsigned long view_top_minus_pixel_height;
+    int proj_is_latlong;
+    short PCS;
+    int usgs_drg;
+    char *imagedesc;
 
-
+    usgs_drg = mdf->usgs_drg;              // yes, no, or auto
 
     if (debug_level & 16)
         fprintf(stderr,"%s/%s\n", dir, filenm);
@@ -498,6 +517,25 @@ void draw_geotiff_image_map (Widget w,
 
     xastir_snprintf(file, sizeof(file), "%s/%s", dir, filenm);
 
+    // Create a shorter filename for display (one that fits the
+    // status line more closely).  Subtract the length of the
+    // "Indexing " and/or "Loading " strings as well.
+    if (strlen(filenm) > (41 - 9)) {
+        int avail = 41 - 11;
+        int new_len = strlen(filenm) - avail;
+
+        xastir_snprintf(short_filenm,
+            sizeof(short_filenm),
+            "..%s",
+            &filenm[new_len]);
+    }
+    else {
+        xastir_snprintf(short_filenm,
+            sizeof(short_filenm),
+            "%s",
+            filenm);
+    }
+
     /* Check whether we have an associated *.fgd file.  This
      * file contains the neat-line corner points for USGS DRG
      * maps, which allows us to chop off the map collar.
@@ -674,7 +712,13 @@ void draw_geotiff_image_map (Widget w,
             GTIFPrintDefn (&defn, stdout);
     }
 
- 
+    proj_is_latlong=FALSE;
+    if( !GTIFKeyGet(gtif,ProjectedCSTypeGeoKey, &PCS,0,1))
+      {
+          //fprintf(stderr,"Warning: no PCS in geotiff file %s, assuming map is in lat/lon!\n", filenm);
+	proj_is_latlong=TRUE;
+      }
+
     /* Fetch a few TIFF fields for this image */
     if ( !TIFFGetField (tif, TIFFTAG_IMAGEWIDTH, &width) ) {
         width = 5493;
@@ -686,6 +730,23 @@ void draw_geotiff_image_map (Widget w,
         fprintf(stderr,"No height tag found in file, setting it to 6840\n");
     }
 
+    // If we're autodetecting usgs_drg, check the image description tag
+    // Note, the TIFFGetField doesn't allocate a string, it returns a pointer
+    // to an existing one.  Don't free it!
+    if (usgs_drg == 2) {
+        if ( TIFFGetField (tif, TIFFTAG_IMAGEDESCRIPTION, &imagedesc)) {
+            if (strncasecmp(imagedesc,"USGS GeoTIFF DRG",16) == 0) {
+                usgs_drg = 1;   // Yes
+            } 
+            else {
+                usgs_drg = 0;  // No
+            }
+        }
+        else {
+                usgs_drg = 0;  // No tag, assume not a usgs topo 
+        }
+            
+    }
 
     /*
      * If we don't have an associated .fgd file for this map,
@@ -729,7 +790,7 @@ void draw_geotiff_image_map (Widget w,
                 fprintf(stderr,"(%11.3f,%11.3f)\n", xxx, yyy );
             }
         }
-        if ( GTIFProj4ToLatLong( &defn, 1, &xxx, &yyy ) )   // Do all 4 of these in one call?
+        if ( proj_is_latlong || GTIFProj4ToLatLong( &defn, 1, &xxx, &yyy ) )   // Do all 4 of these in one call?
         {
             if (debug_level & 16) {
                 fprintf(stderr,"  (%s,", GTIFDecToDMS( xxx, "Long", 2 ) );
@@ -751,7 +812,7 @@ void draw_geotiff_image_map (Widget w,
                 fprintf(stderr,"(%11.3f,%11.3f)\n", xxx, yyy );
             }
         }
-        if ( GTIFProj4ToLatLong( &defn, 1, &xxx, &yyy ) )
+        if (  proj_is_latlong || GTIFProj4ToLatLong( &defn, 1, &xxx, &yyy ) )
         {
             if (debug_level & 16) {
                 fprintf(stderr,"  (%s,", GTIFDecToDMS( xxx, "Long", 2 ) );
@@ -772,7 +833,7 @@ void draw_geotiff_image_map (Widget w,
                 fprintf(stderr,"(%11.3f,%11.3f)\n", xxx, yyy );
             }
         }
-        if ( GTIFProj4ToLatLong( &defn, 1, &xxx, &yyy ) )
+        if (  proj_is_latlong || GTIFProj4ToLatLong( &defn, 1, &xxx, &yyy ) )
         {
             if (debug_level & 16) {
                 fprintf(stderr,"  (%s,", GTIFDecToDMS( xxx, "Long", 2 ) );
@@ -793,7 +854,7 @@ void draw_geotiff_image_map (Widget w,
                 fprintf(stderr,"(%11.3f,%11.3f)\n", xxx, yyy );
             }
         }
-        if ( GTIFProj4ToLatLong( &defn, 1, &xxx, &yyy ) )
+        if (  proj_is_latlong || GTIFProj4ToLatLong( &defn, 1, &xxx, &yyy ) )
         {
             if (debug_level & 16) {
                 fprintf(stderr,"  (%s,", GTIFDecToDMS( xxx, "Long", 2 ) );
@@ -1027,7 +1088,10 @@ void draw_geotiff_image_map (Widget w,
     if ( (destination_pixmap == INDEX_CHECK_TIMESTAMPS)
             || (destination_pixmap == INDEX_NO_TIMESTAMPS) ) {
 
-        xastir_snprintf(map_it, sizeof(map_it), langcode ("BBARSTA039"), filenm);
+        xastir_snprintf(map_it,
+            sizeof(map_it),
+            langcode ("BBARSTA039"),
+            short_filenm);
         statusline(map_it,0);       // Indexing ...
 
         // We're indexing only.  Save the extents in the index.
@@ -1035,7 +1099,8 @@ void draw_geotiff_image_map (Widget w,
             south_bounding_wgs84,   // Bottom
             north_bounding_wgs84,   // Top
             west_bounding_wgs84,    // Left
-            east_bounding_wgs84);   // Right
+            east_bounding_wgs84,    // Right
+            0);                     // Default Map Level
 
         //Free any memory used and return
         /* We're finished with the geoTIFF key parser, so get rid of it */
@@ -1047,7 +1112,10 @@ void draw_geotiff_image_map (Widget w,
         return; // Done indexing this file
     }
     else {
-        xastir_snprintf(map_it, sizeof(map_it), langcode ("BBARSTA028"), filenm);
+        xastir_snprintf(map_it,
+            sizeof(map_it),
+            langcode ("BBARSTA028"),
+            short_filenm);
         statusline(map_it,0);       // Loading ...
     }
 
@@ -1150,7 +1218,16 @@ Samples Per Pixel: 1
         || (  bitsPerSample != 8)
         || (   planarConfig != 1) )
     {
-        fprintf(stderr,"*** geoTIFF file %s is not in the proper format.\n", file);
+        fprintf(stderr,"*** geoTIFF file %s is not in the proper format:\n", file);
+        if (samplesPerPixel != 1)
+            fprintf(stderr,"***** Has %d samples per pixel instead of 1\n",
+                    samplesPerPixel);
+        if (bitsPerSample != 8)
+            fprintf(stderr,"***** Has %d bits per sample instead of 8\n",
+                    bitsPerSample);
+        if (planarConfig != 1)
+            fprintf(stderr,"***** Has planarConfig of %d instead of 1\n",
+                    planarConfig);
         fprintf(stderr,"*** Please reformat it and try again.\n");
         XTIFFClose(tif);
         return;
@@ -1185,17 +1262,37 @@ Samples Per Pixel: 1
 
 
     /* Print out the colormap info */
-    //    if (debug_level & 16) {
-    //        int l;
+    //if (debug_level & 16) {
+    //    int l;
     //
-    //        for (l = 0; l < num_colors; l++)
-    //            fprintf(stderr,"   %5u: %5u %5u %5u\n",
-    //                        l,
-    //                        red_orig[l],
-    //                        green_orig[l],
-    //                        blue_orig[l]);
-    //    }
-
+    //    for (l = 0; l < num_colors; l++)
+    //    fprintf(stderr,"   %5u: %5u %5u %5u\n",
+    //    l,
+    //    red_orig[l],
+    //    green_orig[l],
+    //    blue_orig[l]);
+    //}
+    // Example output from a USGS 7.5' map:
+    //
+    //   0:     0     0     0   black
+    //   1: 65280 65280 65280   light grey
+    //   2:     0 38656 41984
+    //   3: 51968     0  5888
+    //   4: 33536 16896  9472   contour lines, brownish-red
+    //   5: 51456 59904 40192   green
+    //   6: 35072 13056 32768   purple?  freeways, some roads
+    //   7: 65280 59904     0
+    //   8: 42752 57856 57856   blue, bodies of water
+    //   9: 65280 47104 47104   red brick color, cities?
+    //  10: 55808 45824 54784   purple.  freeways, some roads
+    //  11: 53504 53504 53504   grey
+    //  12: 52992 41984 36352   contour lines, tan, more dots/less lines
+    //  13:     0     0     0   black, unused slot?
+    //  14:     0     0     0   black, unused slot?
+    //  15:     0     0     0   black, unused slot?
+    //  16:     0     0     0   black, unused slot?
+    // The rest are all 0's.
+    
 
     if (crop_it)    // USGS geoTIFF map
     {
@@ -1234,7 +1331,7 @@ Samples Per Pixel: 1
         yyy = (double)f_NW_y_bounding;
 
         /* Convert lat/long to projected coordinates */
-        if ( GTIFProj4FromLatLong( &defn, 1, &xxx, &yyy ) )     // Do all 4 in one call?
+        if (  proj_is_latlong || GTIFProj4FromLatLong( &defn, 1, &xxx, &yyy ) )     // Do all 4 in one call?
         {
             if (debug_level & 16)
                 fprintf(stderr,"%11.3f,%11.3f\n", xxx, yyy);
@@ -1312,7 +1409,7 @@ Samples Per Pixel: 1
         yyy = (double)f_NE_y_bounding;
 
         /* Convert lat/long to projected coordinates */
-        if ( GTIFProj4FromLatLong( &defn, 1, &xxx, &yyy ) )
+        if (  proj_is_latlong || GTIFProj4FromLatLong( &defn, 1, &xxx, &yyy ) )
         {
             if (debug_level & 16)
                 fprintf(stderr,"%11.3f,%11.3f\n", xxx, yyy);
@@ -1390,7 +1487,7 @@ Samples Per Pixel: 1
         yyy = (double)f_SW_y_bounding;
 
         /* Convert lat/long to projected coordinates */
-        if ( GTIFProj4FromLatLong( &defn, 1, &xxx, &yyy ) )
+        if (  proj_is_latlong || GTIFProj4FromLatLong( &defn, 1, &xxx, &yyy ) )
         {
             if (debug_level & 16)
                 fprintf(stderr,"%11.3f,%11.3f\n", xxx, yyy);
@@ -1468,7 +1565,7 @@ Samples Per Pixel: 1
         yyy = (double)f_SE_y_bounding;
 
         /* Convert lat/long to projected coordinates */
-        if ( GTIFProj4FromLatLong( &defn, 1, &xxx, &yyy ) )
+        if (  proj_is_latlong || GTIFProj4FromLatLong( &defn, 1, &xxx, &yyy ) )
         {
             if (debug_level & 16)
                 fprintf(stderr,"%11.3f,%11.3f\n", xxx, yyy);
@@ -2183,8 +2280,22 @@ right_crop = width - 1;
     // probably).  A higher number means less rows skipped,
     // which improves the look but slows the map drawing down.
     //
+    // This needs modification somewhat when the raster is already in 
+    // lat/long, and the PixelScale is already in degrees per pixel.
+    // The scale_y is the integer number of hundredths of seconds per pixel.
+    // The decision of how many rows to skip can simply be made by converting
+    // the PixelScale to the same units as scale_y, and skipping accordingly.
+
     if (have_PixelScale) {
-        SkipRows = (int)( ( scale_y / ( *PixelScale * 3.15 ) ) + 0.5 );
+        double coef;
+        if (!proj_is_latlong) {
+            coef = 3.15;
+        } 
+        else {
+            coef=100*60*60;  // xastir coords are in 1/100 of a second,
+                             // and lat/lon pixel scales will be in degrees
+        }
+        SkipRows = (int)( ( scale_y / ( *PixelScale * coef ) ) + 0.5 );
         if (SkipRows < 1)
             SkipRows = 1;
         if (SkipRows > (int)(height / 10) )
@@ -2244,6 +2355,22 @@ right_crop = width - 1;
     }
 
 
+    // Set the bit-blitting function to XOR.  This is only used if
+    // DRG_XOR_colors is set to a non-zero value, but we set it here
+    // outside the pixel loop for speed reasons.
+    //
+    if (DRG_XOR_colors) {
+        (void)XSetLineAttributes (XtDisplay (w), gc_tint, 1, LineSolid, CapButt,JoinMiter);
+
+//        (void)XSetForeground (XtDisplay (w), gc_tint, colors[0x27]);  // yellow
+//        (void)XSetForeground (XtDisplay (w), gc_tint, colors[0x0f]); // White
+//        (void)XSetForeground (XtDisplay (w), gc_tint, colors[0x03]); // cyan
+//        (void)XSetForeground (XtDisplay (w), gc_tint, colors[0x06]); // orange
+ 
+        (void)XSetFunction (XtDisplay (da), gc_tint, GXxor);
+    }
+
+
     // Iterate over the rows of interest only.  Using the rectangular
     // top/bottom crop values for these is ok at this point.
     //
@@ -2300,8 +2427,8 @@ right_crop = width - 1;
 
 //WE7U:  This comparison is always false:  current_left is unsigned
 //therefore always positive!
-if (current_left < 0)
-    current_left = 0;
+//if (current_left < 0)
+//    current_left = 0;
 
 if (current_right >= width)
     current_right = width - 1;
@@ -2408,7 +2535,9 @@ if (current_right >= width)
                             || (xastir_current_x > (SE_x_bounding_wgs84 - 25) ) )
                         {
 //WE7U: column is unsigned so "column >= 0" is always true
-                            if ((int)column < bytesPerRow && column >= 0) {
+//                            if ((int)column < bytesPerRow && column >= 0) {
+                            if ((int)column < bytesPerRow) {
+ 
                                 *(imageMemory + column) = 0x01; // Change to White
                             }
                             else {
@@ -2440,11 +2569,64 @@ if (current_right >= width)
                     sxx = (xastir_current_x - x_long_offset) / scale_x;
                     syy = (xastir_total_y   - y_lat_offset ) / scale_y;
 
-                    // Set the color for the pixel
-                    XSetForeground (XtDisplay (w), gc, my_colors[*(imageMemory + column)].pixel);
 
-                    // And draw the pixel
-                    XFillRectangle (XtDisplay (w), pixmap, gc, sxx, syy, stepwc, stephc);
+
+// If we wish some colors to be transparent, we could check for a
+// color match here and refuse to draw those that are on our list of
+// transparent colors.  Might be useful for drawing say contour
+// lines on top of satellite images.  Since the USGS topo's use the
+// same colormap for all of the topo's (I believe that is true) then
+// we could just compare on the index into the array instead of the
+// color contents in the array.
+//
+    // Example output from a USGS 7.5' map:
+    //
+    //   0:     0     0     0   black
+    //   1: 65280 65280 65280   light grey
+    //   2:     0 38656 41984
+    //   3: 51968     0  5888
+    //   4: 33536 16896  9472   contour lines, brownish-red
+    //   5: 51456 59904 40192   green
+    //   6: 35072 13056 32768   purple?  freeways, some roads
+    //   7: 65280 59904     0
+    //   8: 42752 57856 57856   blue, bodies of water
+    //   9: 65280 47104 47104   red brick color, cities?
+    //  10: 55808 45824 54784   purple.  freeways, some roads
+    //  11: 53504 53504 53504   grey
+    //  12: 52992 41984 36352   contour lines, tan, more dots/less lines
+    //  13:     0     0     0   black, unused slot?
+    //  14:     0     0     0   black, unused slot?
+    //  15:     0     0     0   black, unused slot?
+    //  16:     0     0     0   black, unused slot?
+    // The rest are all 0's.
+                    // either show all colors (usgs_drg==0) or show selected
+                    // colors (usgs_drg == 1) 
+                    if ( usgs_drg == 0 ||
+                         (usgs_drg ==1 && (*(imageMemory + column) >= 13
+                          || DRG_show_colors[*(imageMemory + column)] == 1))) {
+
+
+                        // If this is set, use gc_tint for drawing
+                        // with "GXxor" as the bit-blit operation.
+                        //
+                        if (DRG_XOR_colors) {
+
+                            // Draw the pixel using "gc_tint"
+                            // instead of "gc".
+
+                            XSetForeground (XtDisplay (w), gc_tint, my_colors[*(imageMemory + column)].pixel);
+ 
+                            XFillRectangle (XtDisplay (w), pixmap, gc_tint, sxx, syy, stepwc, stephc);
+                        }
+                        else {
+
+                            // Set the color for the pixel
+                            XSetForeground (XtDisplay (w), gc, my_colors[*(imageMemory + column)].pixel);
+
+                            // And draw the pixel
+                            XFillRectangle (XtDisplay (w), pixmap, gc, sxx, syy, stepwc, stephc);
+                        }
+                    }
                 }
             }
         }
