@@ -1,9 +1,9 @@
 /* -*- c-basic-offset: 4; indent-tabs-mode: nil -*-
- * $Id: map_tiger.c,v 1.17 2004/08/19 19:38:16 we7u Exp $
+ * $Id: map_tiger.c,v 1.41 2005/07/22 06:08:22 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2004  The Xastir Group
+ * Copyright (C) 2000-2005  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -68,6 +68,8 @@
 #include "color.h"
 #include "xa_config.h"
 
+#include "map_cache.h"
+
 #define CHECKMALLOC(m)  if (!m) { fprintf(stderr, "***** Malloc Failed *****\n"); exit(0); }
 
 #ifdef HAVE_IMAGEMAGICK
@@ -111,16 +113,7 @@
 #undef XASTIR_PACKAGE_VERSION
 #endif // HAVE_IMAGEMAGICK
 
-#ifdef HAVE_LIBCURL
-#include <curl/curl.h>
-#include <curl/types.h>
-#include <curl/easy.h>
 
-struct FtpFile {
-  char *filename;
-  FILE *stream;
-};
-#endif  // HAVE_LIBCURL
 
 extern int npoints;		/* tsk tsk tsk -- globals */
 extern int mag;
@@ -129,6 +122,114 @@ extern int mag;
 
 
 
+void get_tiger_local_file(char * local_filename, char * fileimg){
+#ifdef HAVE_IMAGEMAGICK
+
+//  char local_filename[MAX_FILENAME];
+
+//    char fileimg[MAX_FILENAME];     // Ascii name of image file, read from GEO file
+
+
+    time_t query_start_time, query_end_time; 
+
+#ifdef USE_MAP_CACHE 
+    int map_cache_return = 1; // Default = cache miss
+#endif  // USE_MAP_CACHE
+
+
+    if (debug_level & 512) {
+        query_start_time=time(&query_start_time); 
+    }
+
+
+#ifdef USE_MAP_CACHE 
+
+    if (!map_cache_fetch_disable) {
+
+        // Delete old copy from the cache
+        if (map_cache_fetch_disable && fileimg[0] != '\0') {
+            if (map_cache_del(fileimg)) {
+                if (debug_level & 512) {
+                    fprintf(stderr,"Couldn't delete old map from cache\n");
+                }
+            }
+        }
+
+set_dangerous("map_tiger: map_cache_get");
+	map_cache_return = map_cache_get(fileimg,local_filename); 
+clear_dangerous();
+    }
+
+    if (debug_level & 512) {
+            fprintf(stderr,"map_cache_return: <%d> bytes returned: %d\n",
+                map_cache_return,
+                (int) strlen(local_filename));
+    }
+   
+    if (map_cache_return != 0 ) {
+
+set_dangerous("map_tiger: map_cache_fileid");
+        xastir_snprintf(local_filename,
+            MAX_FILENAME,           // hardcoded to avoid sizeof()
+            "%s/map_%s.%s",
+            get_user_base_dir("map_cache"),
+            map_cache_fileid(),
+            "gif");
+clear_dangerous();
+
+#else   // USE_MAP_CACHE
+
+    xastir_snprintf(local_filename,
+        MAX_FILENAME,               // hardcoded to avoid sizeof()
+        "%s/map.%s",
+         get_user_base_dir("tmp"),
+        "gif");
+
+#endif  // USE_MAP_CACHE
+
+
+    // Erase any previously existing local file by the same name.
+    // This avoids the problem of having an old map image here and
+    // the code trying to display it when the download fails.
+
+    unlink( local_filename );
+
+    if (fetch_remote_file(fileimg, local_filename)) {
+        // Had trouble getting the file.  Abort.
+        return;
+    }
+
+    // For debugging the MagickError/MagickWarning segfaults.
+    //system("cat /dev/null >/var/tmp/xastir_hacker_map.gif");
+   
+ 
+#ifdef USE_MAP_CACHE
+
+set_dangerous("map_tiger: map_cache_put");
+	map_cache_put(fileimg,local_filename); 
+clear_dangerous();
+
+        } // end if is cached  DHBROWN
+
+#endif // MAP_CACHE
+
+
+    if (debug_level & 512) {
+        fprintf (stderr, "Fetch or query took %d seconds\n", 
+            (int) (time(&query_end_time) - query_start_time)); 
+    }
+
+    // Set permissions on the file so that any user can overwrite it.
+    chmod(local_filename, 0666);
+
+#endif  //HAVE_IMAGEMAGICK
+
+} // end get_tiger_local_file
+
+
+
+
+
 /**********************************************************
  * draw_tiger_map()
  * N0VH
@@ -136,8 +237,10 @@ extern int mag;
 #ifdef HAVE_IMAGEMAGICK
 void draw_tiger_map (Widget w,
         char *filenm,
-        int destination_pixmap) {
+        int destination_pixmap,
+        int nocache) {  // For future implementation of a "refresh cached map" option
     char file[MAX_FILENAME];        // Complete path/name of image file
+    char short_filenm[MAX_FILENAME];
     FILE *f;                        // Filehandle of image file
     char fileimg[MAX_FILENAME];     // Ascii name of image file, read from GEO file
     char tigertmp[MAX_FILENAME*2];  // Used for putting together the tigermap query
@@ -177,6 +280,7 @@ void draw_tiger_map (Widget w,
     long scale_x0;                  // at widest map area
 
     char local_filename[MAX_FILENAME];
+    
     ExceptionInfo exception;
     Image *image;
     ImageInfo *image_info;
@@ -185,16 +289,6 @@ void draw_tiger_map (Widget w,
     IndexPacket *index_pack;
     int l;
     XColor my_colors[256];
-#ifdef HAVE_LIBCURL
-    CURL *curl;
-    CURLcode res;
-    char curlerr[CURL_ERROR_SIZE];
-    struct FtpFile ftpfile;
-#else   // HAVE_LIBCURL
-#ifdef HAVE_WGET
-    char tempfile[MAX_FILENAME];
-#endif  // HAVE_WGET
-#endif  // HAVE_LIBCURL
     double left, right, top, bottom, map_width, map_height;
     double lat_center  = 0;
     double long_center = 0;
@@ -204,11 +298,36 @@ void draw_tiger_map (Widget w,
     int geo_image_width;        // Image width  from GEO file
     int geo_image_height;       // Image height from GEO file
 
+    // initialize this
+    local_filename[0]='\0';
 
-    xastir_snprintf(map_it, sizeof(map_it), langcode ("BBARSTA028"), "tigermap");
+    // Create a shorter filename for display (one that fits the
+    // status line more closely).  Subtract the length of the
+    // "Indexing " and/or "Loading " strings as well.
+    if (strlen(filenm) > (41 - 9)) {
+        int avail = 41 - 11;
+        int new_len = strlen(filenm) - avail;
+
+        xastir_snprintf(short_filenm,
+            sizeof(short_filenm),
+            "..%s",
+            &filenm[new_len]);
+    }
+    else {
+        xastir_snprintf(short_filenm,
+            sizeof(short_filenm),
+            "%s",
+            filenm);
+    }
+
+    xastir_snprintf(map_it,
+        sizeof(map_it),
+        langcode ("BBARSTA028"),
+        short_filenm);
     statusline(map_it,0);       // Loading ...
 
 
+        
     // Check whether we're indexing or drawing the map
     if ( (destination_pixmap == INDEX_CHECK_TIMESTAMPS)
             || (destination_pixmap == INDEX_NO_TIMESTAMPS) ) {
@@ -220,10 +339,14 @@ void draw_tiger_map (Widget w,
             64800000l,      // Bottom
             0l,             // Top
             0l,             // Left
-            129600000l);    // Right
+            129600000l,     // Right
+            0);             // Default Map Level
 
         // Update statusline
-        xastir_snprintf(map_it, sizeof(map_it), langcode ("BBARSTA039"), filenm);
+        xastir_snprintf(map_it,
+            sizeof(map_it),
+            langcode ("BBARSTA039"),
+            short_filenm);
         statusline(map_it,0);       // Loading/Indexing ...
 
         return; // Done indexing this file
@@ -358,94 +481,29 @@ void draw_tiger_map (Widget w,
           fprintf(stderr,"ftp or http file: %s\n", fileimg);
     }
 
-    xastir_snprintf(local_filename,
-        sizeof(local_filename),
-        "%s/map.%s",
-        get_user_base_dir("tmp"),
-        "gif");
 
-    // Erase any previously existing local file by the same name.
-    // This avoids the problem of having an old map image here and
-    // the code trying to display it when the download fails.
-    unlink( local_filename );
+// Hopefully this will eventually allow us to get maps in the background
+//    while (sometimeout !=0 && local_filename[0]==NULL){
 
-    HandlePendingEvents(app_context);
-    if (interrupt_drawing_now) {
-        // Update to screen
-        (void)XCopyArea(XtDisplay(da),pixmap,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
-        return;
-    }
+    if  (local_filename[0]=='\0' ){
 
-
-#ifdef HAVE_LIBCURL
-    curl = curl_easy_init();
-
-    if (curl) { 
-
-        /* verbose debug is keen */
-      //        curl_easy_setopt(curl, CURLOPT_VERBOSE, TRUE);
-        curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, curlerr);
-
-        /* write function */
-        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, curl_fwrite);
-
-        /* download from fileimg */
-        curl_easy_setopt(curl, CURLOPT_URL, fileimg);
-
-        /* save as local_filename */
-        ftpfile.filename = local_filename;
-        ftpfile.stream = NULL;
-        curl_easy_setopt(curl, CURLOPT_FILE, &ftpfile);    
-         
-        res = curl_easy_perform(curl);
-
-        curl_easy_cleanup(curl);
-
-        if (CURLE_OK != res) {
-            fprintf(stderr, "curl told us %d\n", res);
-            fprintf(stderr, "curlerr is %s\n", curlerr);
+        if (debug_level & 512 ) { 
+            fprintf(stderr,"tiger_local_file=<%s>\n",local_filename);
         }
 
-        if (ftpfile.stream)
-            fclose(ftpfile.stream);
-
-        // Return if we had trouble
-        if (CURLE_OK != res) {
+        HandlePendingEvents(app_context);
+        if (interrupt_drawing_now) {
+            // Update to screen
+            (void)XCopyArea(XtDisplay(da),pixmap,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
             return;
         }
 
-    } else { 
-        fprintf(stderr,"Couldn't download the Tigermap image\n");
-        return;
+        get_tiger_local_file(local_filename,fileimg); 
+
     }
-#else   // HAVE_LIBCURL
-#ifdef HAVE_WGET
-    xastir_snprintf(tempfile, sizeof(tempfile),
-        "%s --server-response --timestamping --tries=1 --timeout=%d --output-document=%s \'%s\' 2> /dev/null\n",
-        WGET_PATH,
-        tigermap_timeout,
-        local_filename,
-        fileimg);
 
-    if (debug_level & 512)
-       fprintf(stderr,"%s",tempfile);
+// whackadoodle
 
-    if (system(tempfile)) {   // Go get the file
-       fprintf(stderr,"Couldn't download the Tigermap image\n");
-       return;
-    }
-#else   // HAVE_WGET
-        fprintf(stderr,"libcurl or 'wget' not installed.  Can't download image\n");
-#endif  // HAVE_WGET
-#endif  // HAVE_LIBCURL
-
-
-    // For debugging the MagickError/MagickWarning segfaults.
-    //system("cat /dev/null >/var/tmp/xastir_hacker_map.gif");
-
-
-    // Set permissions on the file so that any user can overwrite it.
-    chmod(local_filename, 0666);
 
     // Tell ImageMagick where to find it
     xastir_snprintf(file,
@@ -801,6 +859,3 @@ void draw_tiger_map (Widget w,
 }
 #endif //HAVE_IMAGEMAGICK
 ///////////////////////////////////////////// End of Tigermap code ///////////////////////////////////////
-
-
-
