@@ -1,9 +1,9 @@
 /*
- * $Id: messages.c,v 1.36 2004/08/24 04:47:53 we7u Exp $
+ * $Id: messages.c,v 1.56 2005/07/06 15:22:47 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2004  The Xastir Group
+ * Copyright (C) 2000-2005  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -35,7 +35,11 @@
 #include <signal.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <strings.h>
+#ifdef HAVE_STRING_H
+#  include <string.h>
+#else
+#  include <strings.h>
+#endif
 #include <ctype.h>
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -179,13 +183,20 @@ void group_build_list(char *filename) {
             else
                 while ((i = fgetc(f)) != EOF && i != '\n'); // clean-up after long group name
 
+	// check for DOS EOL markup!
+            if ((ptr = strchr(&group_data_list[group_data_count*10], '\r')))
+                *ptr = '\0';
             if (group_data_list[group_data_count*10])
                 group_data_count++;
         }
     }
     (void)fclose(f);
     qsort(group_data_list, (size_t)group_data_count, 10, group_comp);
-    if (debug_level & 2) for (i = 0; i < group_data_count; i++) printf("Group %2d: %s\n", i, &group_data_list[i*10]);
+
+    if (debug_level & 2) {
+        for (i = 0; i < group_data_count; i++)
+            fprintf(stderr,"Group %2d: %s\n", i, &group_data_list[i*10]);
+    }
 }
 
 
@@ -257,20 +268,33 @@ end_critical_section(&send_message_dialog_lock, "messages.c:look_for_open_group_
 
 
 
+// What we wish to do here:  Check for an active Send Message dialog
+// that contains the callsign of interest.  If one doesn't exist,
+// create one and pop it up.  We don't want to do this for duplicate
+// message lines or duplicate acks for any particular QSO.  To do so
+// would cause the Send Message dialog to pop up on every such
+// received message, which is VERY annoying (that was the default in
+// Xastir for years, and nobody liked it!).
+//
 int check_popup_window(char *from_call_sign, int group) {
     int i,found,j,ret;
     char temp1[MAX_CALLSIGN+1];
     char *temp_ptr;
 
 
+//fprintf(stderr,"\tcheck_popup_window()\n");
+
     ret =- 1;
     found =- 1;
 
 begin_critical_section(&send_message_dialog_lock, "messages.c:check_popup_window" );
 
+    // Check for an already-created dialog for talking to this
+    // particular call_sign.
+    //
     for (i = 0; i < MAX_MESSAGE_WINDOWS; i++) {
-        /* find station  */
-        if (mw[i].send_message_dialog != NULL) {
+
+        if (mw[i].send_message_dialog != NULL) { // If dialog created
 
             temp_ptr = XmTextFieldGetString(mw[i].send_message_call_data);
             xastir_snprintf(temp1,
@@ -281,6 +305,10 @@ begin_critical_section(&send_message_dialog_lock, "messages.c:check_popup_window
 
             /*fprintf(stderr,"Looking at call <%s> for <%s>\n",temp1,from_call_sign);*/
             if (strcasecmp(temp1, from_call_sign) == 0) {
+                // Found a call_sign match in a Send Message dialog!
+
+//fprintf(stderr,"\tFound a Send_message dialog match\n");
+
                 found = i;
                 break;
             }
@@ -289,9 +317,14 @@ begin_critical_section(&send_message_dialog_lock, "messages.c:check_popup_window
 
 end_critical_section(&send_message_dialog_lock, "messages.c:check_popup_window" );
 
+    // If found == -1 at this point, we haven't found a Send Message
+    // dialog that contains the call_sign of interest.
+    //
     if (found == -1 && (group == 2 || group_active(from_call_sign))) {
         /* no window found Open one! */
 
+//fprintf(stderr,"\tNo Send Message dialog found, creating one\n");
+
 begin_critical_section(&send_message_dialog_lock, "messages.c:check_popup_window2" );
 
         i=-1;
@@ -305,22 +338,38 @@ begin_critical_section(&send_message_dialog_lock, "messages.c:check_popup_window
 end_critical_section(&send_message_dialog_lock, "messages.c:check_popup_window2" );
 
         if (i!=-1) {
-        if (group == 1) {
-        temp1[0] = '*';
-        temp1[1] = '\0';
-        } else
-            temp1[0] = '\0';
+
+            if (group == 1) {
+                temp1[0] = '*';
+                temp1[1] = '\0';
+            }
+            else {
+                temp1[0] = '\0';
+            }
+
             strncat(temp1,
                 from_call_sign,
                 sizeof(temp1) - strlen(temp1));
+
             Send_message(appshell, temp1, NULL);
+
             update_messages(1);
+
             ret=i;
-        } else
+        }
+        else {
             fprintf(stderr,"No open windows!\n");
-    } else
+        }
+    }
+    else {
         /* window open! */
+        // Pop it up
         ret=found;
+    }
+
+    if (found != -1) {  // Already have a window
+        XtPopup(mw[i].send_message_dialog,XtGrabNone);
+    }
 
     return(ret);
 }
@@ -361,6 +410,15 @@ void clear_outgoing_messages_to(char *callsign) {
         // If it matches the callsign we're talking to
         if (strcasecmp(message_pool[ii].to_call_sign,callsign) == 0) {
 
+            // Record a fake ack and add "*CANCELLED*" to the
+            // message.  This will be displayed in the Send Message
+            // dialog.
+            msg_record_ack(message_pool[ii].to_call_sign,
+                message_pool[ii].from_call_sign,
+                message_pool[ii].seq,
+                0,  // Not a timeout
+                1); // Record a cancel
+ 
             // Clear it out.
             message_pool[ii].active=MESSAGE_CLEAR;
             message_pool[ii].to_call_sign[0] = '\0';
@@ -378,6 +436,86 @@ void clear_outgoing_messages_to(char *callsign) {
 
 
 
+// Change path on all pending transmit messages that are from us and
+// to the callsign listed.
+//
+void change_path_outgoing_messages_to(char *callsign, char *new_path) {
+    int ii;
+    char my_callsign[20];
+
+
+//fprintf(stderr,
+//    "Changing all outgoing msgs to %s to new path: %s\n",
+//    callsign,
+//    new_path);
+
+    xastir_snprintf(my_callsign,
+        sizeof(my_callsign),
+        "%s",
+        callsign);
+
+    remove_trailing_spaces(my_callsign);
+
+    // Run through the entire outgoing message queue
+    for (ii = 0; ii < MAX_OUTGOING_MESSAGES; ii++) {
+
+        if (message_pool[ii].active == MESSAGE_ACTIVE) {
+ 
+//fprintf(stderr,"\t'%s'\n\t'%s'\n",
+//    message_pool[ii].to_call_sign,
+//    my_callsign);
+
+            // If it matches the callsign we're talking to
+            if (strcasecmp(message_pool[ii].to_call_sign,my_callsign) == 0) {
+
+//fprintf(stderr,"\tFound an outgoing queued msg to change path on.\n");
+
+                xastir_snprintf(message_pool[ii].path,
+                    sizeof(message_pool[ii].path),
+                    "%s",
+                    new_path);
+            }
+        }
+    }
+}
+
+
+
+
+
+time_t last_check_and_transmit = (time_t)0l;
+
+
+// Kick the interval timer back to 7 and tries back to 1 for
+// messages in this QSO.  Used to get a QSO going again when the
+// interval timer has gotten large, but the message is important to
+// get through quickly.
+//
+void kick_outgoing_timer(char *callsign) {
+    int ii;
+
+
+//    fprintf(stderr,"Callsign: %s\n", callsign);
+
+    // Run through the entire outgoing message queue
+    for (ii = 0; ii < MAX_OUTGOING_MESSAGES; ii++) {
+
+        // If it matches the callsign we're talking to
+        if (strcasecmp(message_pool[ii].to_call_sign,callsign) == 0) {
+            message_pool[ii].next_time = (time_t)7l;
+            message_pool[ii].tries = 0;
+            message_pool[ii].active_time = (time_t)0l;
+        }
+    }
+
+    // Cause the transmit routine to get called again
+    last_check_and_transmit = (time_t)0l;
+}
+
+
+
+
+
 void reset_outgoing_messages(void) {
     int i;
 
@@ -412,26 +550,85 @@ end_critical_section(&send_message_dialog_lock, "messages.c:clear_outgoing_messa
 
 
 
+// Bumps message sequence ID up to the next value.
+//
+// Roll over message_counter if we hit the max.  Now with Reply/Ack
+// protocol the max is only two characters worth.  We changed to
+// sending the sequence number in Base-?? format in order to get
+// more range from the 2-character variable.
+//
+void bump_message_counter(void) {
+
+    message_counter[2] = '\0';  // Terminate at 2 chars
+
+    // Increment the least significant digit
+    message_counter[1]++;
+
+    // Span the gaps between the correct ranges
+    if (message_counter[1] == ':')
+        message_counter[1] = 'A';
+
+    if (message_counter[1] == '[')
+        message_counter[1] = 'a';
+
+    if (message_counter[1] == '{') {
+        message_counter[1] = '0';
+        message_counter[0]++;   // Roll over to next char
+    }
+
+    // Span the gaps between the correct ranges
+    if (message_counter[0] == ':')
+        message_counter[0] = 'A';
+
+    if (message_counter[0] == '[')
+        message_counter[0] = 'a';
+
+    if (message_counter[0] == '{') {
+        message_counter[0] = '0';
+        message_counter[0]++;
+    }
+}
+
+
+
+
+
 // Adds a message to the outgoing message queue.  Doesn't actually
 // cause a transmit.  "check_and_transmit_messages()" is the
 // function which actually gets things moving.
+//
+// We also stuff the message into the main message queue so that the
+// queued messages will appear in the Send Message box.
+//
 void output_message(char *from, char *to, char *message, char *path) {
     int ok,i,j;
     char message_out[MAX_MESSAGE_OUTPUT_LENGTH+1];
     int last_space, message_ptr, space_loc;
     int wait_on_first_ack;
     int error;
+    long record;
+
 
     message_ptr=0;
     last_space=0;
     ok=0;
     error=0;
+
     if (debug_level & 2)
         fprintf(stderr,"Output Message from <%s>  to <%s>\n",from,to);
 
+    // Repeat until we process the entire message.  We'll process it
+    // a chunk at a time, size of chunk to correspond to max APRS
+    // message line length.
+    //
     while (!error && (message_ptr < (int)strlen(message))) {
         ok=0;
         space_loc=0;
+
+        // Break a long message into smaller chunks that can be
+        // processed into APRS messages.  Break at a space character
+        // if possible.
+        //
         for (j=0;j<MAX_MESSAGE_OUTPUT_LENGTH;j++) {
             if(message[j+message_ptr] != '\0') {
                 if(message[j+message_ptr]==' ') {
@@ -452,10 +649,14 @@ void output_message(char *from, char *to, char *message, char *path) {
                 last_space=strlen(message)+1;
             }
         }
+
+//fprintf(stderr,"message_out: %s\n", message_out);
+
         if (debug_level & 2)
             fprintf(stderr,"MESSAGE <%s> %d %d\n",message_out,message_ptr,last_space);
 
         message_ptr=last_space;
+
         /* check for others in the queue */
         wait_on_first_ack=0;
         for (i=0; i<MAX_OUTGOING_MESSAGES; i++) {
@@ -466,46 +667,15 @@ void output_message(char *from, char *to, char *message, char *path) {
                 i=MAX_OUTGOING_MESSAGES+1;  // Done with loop
             }
         }
+
         for (i=0; i<MAX_OUTGOING_MESSAGES && !ok ;i++) {
             /* Check for clear position*/
             if (message_pool[i].active==MESSAGE_CLEAR) {
                 /* found a spot */
                 ok=1;
 
-                // Roll over message_counter if we hit the max.  Now
-                // with Reply/Ack protocol the max is only two
-                // characters worth.  We changed to sending the
-                // sequence number in Base-?? format in order to get
-                // more range from the 2-character variable.
-
-                message_counter[2] = '\0';  // Terminate at 2 chars
-
-                // Increment the least significant digit
-                message_counter[1]++;
-
-                // Span the gaps between the correct ranges
-                if (message_counter[1] == ':')
-                    message_counter[1] = 'A';
-
-                if (message_counter[1] == '[')
-                    message_counter[1] = 'a';
-
-                if (message_counter[1] == '{') {
-                    message_counter[1] = '0';
-                    message_counter[0]++;   // Roll over to next char
-                }
-
-                // Span the gaps between the correct ranges
-                if (message_counter[0] == ':')
-                    message_counter[0] = 'A';
-
-                if (message_counter[0] == '[')
-                    message_counter[0] = 'a';
-
-                if (message_counter[0] == '{') {
-                    message_counter[0] = '0';
-                    message_counter[0]++;
-                }
+                // Increment the message sequence ID variable
+                bump_message_counter();
 
 
 // Note that Xastir's messaging can lock up if we do a rollover and
@@ -563,6 +733,36 @@ void output_message(char *from, char *to, char *message, char *path) {
                     message_pool[i].tries = 0;
                 else
                     message_pool[i].tries = MAX_TRIES-1;
+
+                // Cause the message to get added to the main
+                // message queue as well, with the proper sequence
+                // number, so queued messages will appear in the
+                // Send Message box as unacked messages.
+                //
+
+// We must get rid of the lock we already have for a moment, as
+// update_messages(), which is called by msg_data_add(), also snags
+// this lock.
+end_critical_section(&send_message_dialog_lock, "db.c:update_messages" );
+
+                msg_data_add(to,
+                    from,
+                    message_out,
+                    message_pool[i].seq,
+                    MESSAGE_MESSAGE,
+                    'L',    // From the Local system
+                    &record);
+/*
+                fprintf(stderr,"msg_data_add %s %s %s %s\n",
+                    to,
+                    from,
+                    message_out,
+                    message_pool[i].seq);
+*/
+
+// Regain the lock we had before
+begin_critical_section(&send_message_dialog_lock, "db.c:update_messages" );
+
             }
         }
         if(!ok) {
@@ -634,7 +834,197 @@ void transmit_message_data(char *to, char *message, char *path) {
 
 
 
-time_t last_check_and_transmit = (time_t)0l;
+// The below variables and functions implement the capability to
+// schedule ACK's some number of seconds out from the current time.
+// We use it to schedule duplicate ACK's at 30/60/120 seconds out,
+// but only if we see duplicate message lines from remote stations.
+//
+// Create a struct to hold the delayed ack's.
+typedef struct _delayed_ack_record {
+    char to_call_sign[MAX_CALLSIGN+1];
+    char message_line[MAX_MESSAGE_OUTPUT_LENGTH+1];
+    char path[200];
+    time_t active_time;
+    struct _delayed_ack_record *next;
+} delayed_ack_record, *delayed_ack_record_p;
+
+// And a pointer to a list of them.
+    delayed_ack_record_p delayed_ack_list_head = NULL;
+
+
+void transmit_message_data_delayed(char *to, char *message,
+                                   char *path, time_t when) {
+    delayed_ack_record_p ptr = delayed_ack_list_head;
+
+
+    // We need to run down the current list looking for any records
+    // that are identical and within 30 seconds time-wise of this
+    // one.  If so, don't allocate a new record.  This keeps the
+    // dupes down on transmit so that at the most we transmit one
+    // ack per 30 seconds per QSO, except of course for real-time
+    // ack's which don't go through this function.
+
+    // Run through the queue and check each record
+    while (ptr != NULL) {
+
+        if ( strcmp(ptr->to_call_sign,to) == 0
+                && strcmp(ptr->message_line,message) == 0 ) {
+
+            //
+            // We have matches on call_sign and message.  Check the
+            // time next.
+            //
+            if (abs(when - ptr->active_time) < 30) {
+                //
+                // We're within 30 seconds of an identical ack.
+                // Drop this new one (don't add it).
+                //
+
+//fprintf(stderr,"Dropping delayed ack: Too close timewise to another: %s, %s\n",
+//    to, message);
+
+                return; // Don't allocate new record on queue
+            }
+        }
+        ptr = ptr->next;
+    }
+
+    // If we made it to here, there aren't any queued ACK's that are
+    // close enough in time to drop this new one.  Add it to the
+    // queue.
+
+//fprintf(stderr, "Queuing ACK for delayed transmit: %s, %s\n",
+//    to, message);
+
+    // Allocate a record to hold it
+    ptr = (delayed_ack_record_p)malloc(sizeof(delayed_ack_record));
+
+    // Fill in the record
+    xastir_snprintf(ptr->to_call_sign,
+        sizeof(ptr->to_call_sign),
+        "%s",
+        to);
+
+    xastir_snprintf(ptr->message_line,
+        sizeof(ptr->message_line),
+        "%s",
+        message);
+
+    if (path == NULL) {
+        ptr->path[0] = '\0';
+    }
+    else {
+        xastir_snprintf(ptr->path,
+            sizeof(ptr->path),
+            "%s",
+            path);
+    }
+
+    ptr->active_time = when;
+
+    // Add the record to the head of the list
+    ptr->next = delayed_ack_list_head;
+    delayed_ack_list_head = ptr;
+}
+
+
+
+
+
+time_t delayed_transmit_last_check = (time_t)0;
+
+
+void check_delayed_transmit_queue(void) {
+    delayed_ack_record_p ptr = delayed_ack_list_head;
+    int active_records = 0;
+
+
+    // Skip this function if we did it during this second already.
+    if (delayed_transmit_last_check == sec_now()) {
+        return;
+    }
+    delayed_transmit_last_check = sec_now();
+
+//fprintf(stderr, "Checking delayed TX queue for something to transmit.\n");
+//fprintf(stderr, ".");
+
+    // Run down the linked list checking every record.
+    while (ptr != NULL) {
+        if (ptr->active_time != 0) {   // Active record
+            char new_path[MAX_LINE_SIZE+1];
+
+
+//fprintf(stderr, "Found active record\n");
+
+            active_records++;
+
+
+            // Check for a custom path having been set in the Send
+            // Message dialog.  If so, use this for our outgoing
+            // path instead and reset all of the queued message
+            // paths to this station to this new path.
+            //
+            get_send_message_path(ptr->to_call_sign,
+                new_path,
+                sizeof(new_path));
+
+            if (new_path[0] != '\0'
+                    && strcmp(new_path, ptr->path) != 0) {
+
+                // We have a custom path set which is different than
+                // the path saved with the outgoing message.  Change
+                // the path to match the new path.
+                //
+//fprintf(stderr,
+//    "Changing queued ack's to new path: %s\n",
+//    new_path);
+
+                xastir_snprintf(ptr->path,
+                    sizeof(ptr->path),
+                    "%s",
+                    new_path);
+            }
+
+
+            if (ptr->active_time <= sec_now()) {
+                // Transmit it
+//fprintf(stderr,"Found something delayed to transmit!  %ld\n",sec_now());
+
+                if (ptr->path[0] == '\0') {
+                    transmit_message_data(ptr->to_call_sign,
+                        ptr->message_line,
+                        NULL);
+                }
+                else {
+                    transmit_message_data(ptr->to_call_sign,
+                        ptr->message_line,
+                        ptr->path);
+                }
+                
+                ptr->active_time = (time_t)0;
+            }
+        }
+        ptr = ptr->next;
+    } 
+
+    // Check if entire list contains inactive records.  If so,
+    // delete the list.
+    //
+    if (!active_records && (delayed_ack_list_head != NULL)) {
+        // No active records, but the list isn't empty.  Reclaim the
+        // records in the list.
+        while (delayed_ack_list_head != NULL) {
+            ptr = delayed_ack_list_head->next;
+            free(delayed_ack_list_head);
+//fprintf(stderr,"Free'ing delayed_ack record\n");
+            delayed_ack_list_head = ptr;
+        }
+    }
+}
+
+
+
+
 
 void check_and_transmit_messages(time_t time) {
     int i;
@@ -650,64 +1040,122 @@ void check_and_transmit_messages(time_t time) {
 
     for (i=0; i<MAX_OUTGOING_MESSAGES;i++) {
         if (message_pool[i].active==MESSAGE_ACTIVE) {
-            if (message_pool[i].wait_on_first_ack!=1) {
+            if (message_pool[i].wait_on_first_ack!=1) { // Tx only if 0
                 if (message_pool[i].active_time < time) {
                     char *last_ack_ptr;
                     char last_ack[5+1];
 
-                    /* sending message let the tnc and net transmits check to see if we should */
-                    if (debug_level & 2)
-                        fprintf(stderr,"Time %ld Active time %ld next time %ld\n",(long)time,(long)message_pool[i].active_time,(long)message_pool[i].next_time);
 
-                    if (debug_level & 2)
-                        fprintf(stderr,"Send message#%d to <%s> from <%s>:%s-%s\n",
-                            message_pool[i].tries,message_pool[i].to_call_sign,message_pool[i].from_call_sign,message_pool[i].message_line,message_pool[i].seq);
+                    if (message_pool[i].tries < MAX_TRIES) {
+                        char new_path[MAX_LINE_SIZE+1];
 
-                    pad_callsign(to_call,message_pool[i].to_call_sign);
+ 
+                        /* sending message let the tnc and net transmits check to see if we should */
+                        if (debug_level & 2)
+                            fprintf(stderr,
+                                "Time %ld Active time %ld next time %ld\n",
+                                (long)time,
+                                (long)message_pool[i].active_time,
+                                (long)message_pool[i].next_time);
 
-                    // Add Leading ":" as per APRS Spec.
-                    // Add trailing '}' to signify that we're
-                    // Reply/Ack protocol capable.
-                    last_ack_ptr = get_most_recent_ack(to_call);
-                    if (last_ack_ptr != NULL)
-                        xastir_snprintf(last_ack,
-                            sizeof(last_ack),
-                            "%s",
-                            last_ack_ptr);
-                    else
-                        last_ack[0] = '\0';
+                        if (debug_level & 2)
+                            fprintf(stderr,"Send message#%d to <%s> from <%s>:%s-%s\n",
+                                message_pool[i].tries,
+                                message_pool[i].to_call_sign,
+                                message_pool[i].from_call_sign,
+                                message_pool[i].message_line,
+                                message_pool[i].seq);
+
+                        pad_callsign(to_call,message_pool[i].to_call_sign);
+
+                        // Add Leading ":" as per APRS Spec.
+                        // Add trailing '}' to signify that we're
+                        // Reply/Ack protocol capable.
+                        last_ack_ptr = get_most_recent_ack(to_call);
+                        if (last_ack_ptr != NULL)
+                            xastir_snprintf(last_ack,
+                                sizeof(last_ack),
+                                "%s",
+                                last_ack_ptr);
+                        else
+                            last_ack[0] = '\0';
                         
-                    xastir_snprintf(temp, sizeof(temp), ":%s:%s{%s}%s",
-                            to_call,
-                            message_pool[i].message_line,
-                            message_pool[i].seq,
-                            last_ack);
+                        xastir_snprintf(temp, sizeof(temp), ":%s:%s{%s}%s",
+                                to_call,
+                                message_pool[i].message_line,
+                                message_pool[i].seq,
+                                last_ack);
 
-                    if (debug_level & 2)
-                        fprintf(stderr,"MESSAGE OUT>%s<\n",temp);
+                        if (debug_level & 2)
+                            fprintf(stderr,"MESSAGE OUT>%s<\n",temp);
 
-                    transmit_message_data(message_pool[i].to_call_sign,temp,message_pool[i].path);
 
-                    message_pool[i].active_time = time + message_pool[i].next_time;
+                        // Check for a custom path having been set
+                        // in the Send Message dialog.  If so, use
+                        // this for our outgoing path instead and
+                        // reset all of the queued message paths to
+                        // this station to this new path.
+                        //
+                        get_send_message_path(to_call,
+                            new_path,
+                            sizeof(new_path));
 
-                    //fprintf(stderr,"%d\n",(int)message_pool[i].next_time);
+//fprintf(stderr,"get_send_message_path(%s) returned: %s\n",to_call,new_path);
+ 
+                        if (new_path[0] != '\0'
+                                && strcmp(new_path,message_pool[i].path) != 0) {
 
-                    // Start at 7 seconds for the interval.  Add 5 seconds to the
-                    // interval each retry until we hit 90 seconds.  90 second
-                    // intervals until retry 30, then start adding 30 seconds to
-                    // the interval each time until we get to a 10 minute interval
-                    // rate.  Keep transmitting at 10 minute intervals until we
-                    // hit MAX_TRIES.
+                            // We have a custom path set which is
+                            // different than the path saved with
+                            // the outgoing message.
+                            //
+                            // Change all messages to that callsign
+                            // to match the new path.
+                            // 
+                            change_path_outgoing_messages_to(to_call,new_path);
+                        }
 
-                    // Increase interval by 5 seconds each time
-                    // until we hit 90 seconds
-                    if ( message_pool[i].next_time < (time_t)90l )
-                         message_pool[i].next_time += 5;
 
-                    // Increase the interval by 30 seconds each time
-                    // after we hit 30 retries
-                    if ( message_pool[i].tries >= 30 )
-                        message_pool[i].next_time += 30;
+                        // Transmit the message
+                        transmit_message_data(message_pool[i].to_call_sign,
+                            temp,
+                            message_pool[i].path);
+
+
+                        message_pool[i].active_time = time + message_pool[i].next_time;
+
+                        //fprintf(stderr,"%d\n",(int)message_pool[i].next_time);
+                    }
+
+/*
+fprintf(stderr,
+    "Msg Interval = %3ld seconds or %4.1f minutes\n",
+    message_pool[i].next_time,
+    message_pool[i].next_time / 60.0);
+*/
+
+                    // Record the interval we're using.  Put it with
+                    // the message in the general message pool, so
+                    // that the Send Message dialog can display it.
+                    // It will only display it if the message is
+                    // actively being transmitted.  If it has been
+                    // cancelled, timed out, or hasn't made it to
+                    // the transmit position yet, it won't be shown.
+                    //
+                    msg_record_interval_tries(message_pool[i].to_call_sign,
+                        message_pool[i].from_call_sign,
+                        message_pool[i].seq,
+                        message_pool[i].next_time,  // Interval
+                        message_pool[i].tries);     // Tries
+
+                    // Start at 7 seconds for the interval.  We set
+                    // it to 7 seconds in output_message() above.
+                    // Double the interval each retry until we hit
+                    // 10 minutes.  Keep transmitting at 10 minute
+                    // intervals until we hit MAX_TRIES.
+
+                    // Double the interval between messages
+                    message_pool[i].next_time = message_pool[i].next_time * 2;
 
                     // Limit the max interval to 10 minutes
                     if (message_pool[i].next_time > (time_t)600l)
@@ -742,13 +1190,14 @@ void check_and_transmit_messages(time_t time) {
                             "%s",
                             message_pool[i].to_call_sign);
 
-                        // Record a fake ack and add "TIMEOUT:" to
+                        // Record a fake ack and add "*TIMEOUT*" to
                         // the message.  This will be displayed in
                         // the Send Message dialog.
                         msg_record_ack(temp_to,
                             message_pool[i].from_call_sign,
                             message_pool[i].seq,
-                            1); // "1" specifies a timeout
+                            1,  // "1" specifies a timeout
+                            0); // Not a cancel
 
                         clear_acked_message(temp_to,
                             message_pool[i].from_call_sign,
@@ -790,7 +1239,13 @@ void clear_acked_message(char *from, char *to, char *seq) {
         if (message_pool[i].active==MESSAGE_ACTIVE) {
 
             if (debug_level & 1)
-                fprintf(stderr,"TO <%s> <%s> from <%s> <%s> seq <%s> <%s>\n",to,message_pool[i].to_call_sign,from,message_pool[i].from_call_sign,seq,message_pool[i].seq);
+                fprintf(stderr,
+                    "TO <%s> <%s> from <%s> <%s> seq <%s> <%s>\n",
+                    to,
+                    message_pool[i].to_call_sign,
+                    from,
+                    message_pool[i].from_call_sign,seq,
+                    message_pool[i].seq);
 
             if (strcmp(message_pool[i].to_call_sign,from)==0) {
                 if (debug_level & 1)
@@ -883,4 +1338,3 @@ void send_queued(char *to) {
     }
 }
 
-
