@@ -1,9 +1,9 @@
 /*
- * $Id: messages_gui.c,v 1.28 2004/08/24 04:47:53 we7u Exp $
+ * $Id: messages_gui.c,v 1.43 2005/07/31 06:19:14 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2004  The Xastir Group
+ * Copyright (C) 2000-2005  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -199,63 +199,170 @@ void get_path_data(char *callsign, char *path, int max_length) {
     if (search_station_name(&p_station,callsign,1)) {  // Found callsign
         char new_path[200];
 
-        xastir_snprintf(new_path,sizeof(new_path),p_station->node_path_ptr);
-
-        if(debug_level & 2)
-            fprintf(stderr,"\nPath from %s: %s\n",
-                callsign,
-                new_path);
-
-// We need to chop off the first call, remove asterisks and
-// injection ID's, and reverse the order of the callsigns.  We need
-// to do the same thing in the callback for button_submit_call, so
-// that we get a new path whenever the callsign is changed.  Create
-// a new TextFieldWidget to hold the path info, which gets filled in
-// here (and the callback) but can be changed by the user.  Must
-// find a nice way to use this path from output_my_data() as well.
-
-        reverse_path(new_path);
-
-        if (debug_level & 2)
-            fprintf(stderr,"  Path to %s: %s\n",
-                callsign,
-                new_path);
-
-        xastir_snprintf(path,
-            max_length,
-            "%s",
-            new_path);
+        if (p_station->node_path_ptr) {
+            xastir_snprintf(new_path,sizeof(new_path),p_station->node_path_ptr);
+            
+            if(debug_level & 2)
+                fprintf(stderr,"\nPath from %s: %s\n",
+                        callsign,
+                        new_path);
+            
+            // We need to chop off the first call, remove asterisks
+            // and injection ID's, and reverse the order of the
+            // callsigns.  We need to do the same thing in the
+            // callback for button_submit_call, so that we get a new
+            // path whenever the callsign is changed.  Create a new
+            // TextFieldWidget to hold the path info, which gets
+            // filled in here (and the callback) but can be changed by
+            // the user.  Must find a nice way to use this path from
+            // output_my_data() as well.
+            
+            reverse_path(new_path);
+            
+            if (debug_level & 2)
+                fprintf(stderr,"  Path to %s: %s\n",
+                        callsign,
+                        new_path);
+            
+            xastir_snprintf(path,
+                            max_length,
+                            "%s",
+                            new_path);
+        } else {
+            if (debug_level & 2) 
+                fprintf(stderr," Path from %s is (null)\n",callsign);
+            path[0]='\0';
+        }
     }
     else {  // Couldn't find callsign.  It's
             // not in our station database.
         if(debug_level & 2)
             fprintf(stderr,"Path from %s: No Path Known\n",callsign);
-
+        
         path[0] = '\0';
     }
+
 }
 
+
  
 
 
+// Find a custom path set in a Send Message dialog, using the remote
+// callsign as the key.  If no custom path, sets path to '\0'.
+//
+void get_send_message_path(char *callsign, char *path, int path_size) {
+    int ii;
+    int found = -1;
+    char *temp_ptr;
+    char temp1[MAX_LINE_SIZE+1];
+    char my_callsign[20];
+    
+
+    xastir_snprintf(my_callsign,sizeof(my_callsign),"%s",callsign); 
+    remove_trailing_spaces(my_callsign);
+
+//fprintf(stderr,"Looking for %s\n", my_callsign);
+    for(ii = 0; ii < MAX_MESSAGE_WINDOWS; ii++) {
+
+        // find matching callsign
+        if(mw[ii].send_message_dialog != NULL) {
+
+            temp_ptr = XmTextFieldGetString(mw[ii].send_message_call_data);
+            xastir_snprintf(temp1,
+                sizeof(temp1),
+                "%s",
+                temp_ptr);
+            XtFree(temp_ptr);
+
+            (void)to_upper(temp1);
+            if(strcmp(temp1,my_callsign)==0) {
+                found = ii;
+                break;
+            }
+        }
+    }
+
+    if (found == -1) {
+//fprintf(stderr,"Didn't find dialog\n");
+        path[0] = '\0';
+        return;
+    }
+
+    // We have the correct Send Message dialog.  Snag the path.
+    //
+    temp_ptr = XmTextFieldGetString(mw[ii].send_message_path);
+    xastir_snprintf(temp1,
+        sizeof(temp1),
+        "%s",
+        temp_ptr);
+    XtFree(temp_ptr);
+
+    (void)to_upper(temp1);
+    (void)remove_leading_spaces(temp1);
+    (void)remove_trailing_spaces(temp1);
+
+    // Path empty?
+    if (temp1[0] == '\0') {
+//fprintf(stderr,"Didn't find custom path\n");
+        path[0] = '\0';
+        return;
+    }
+
+    // We have a real path!  Stuff it into the path variable.
+    xastir_snprintf(path,
+        path_size,
+        "%s",
+        temp1);
+//fprintf(stderr,"Found custom path: %s\n", path);
+}
+
+
+
+
 
 void Send_message_destroy_shell( /*@unused@*/ Widget widget, XtPointer clientData, /*@unused@*/ XtPointer callData) {
-    int i; 
-
-    i=atoi((char *)clientData);
+    int ii;
+//    char *temp_ptr;
+//    char temp1[MAX_LINE_SIZE+1];
+ 
+    ii=atoi((char *)clientData);
 
 begin_critical_section(&send_message_dialog_lock, "messages_gui.c:Send_message_destroy_shell" );
 
-    if (mw[i].send_message_dialog) {
-        XtPopdown(mw[i].send_message_dialog);
-        XtDestroyWidget(mw[i].send_message_dialog);
+    if (mw[ii].send_message_dialog) {
+
+/*
+        // Check whether the send_message_call_data field has a
+        // custom path entered.
+        temp_ptr = XmTextFieldGetString(mw[ii].send_message_call_data);
+        xastir_snprintf(temp1,
+                sizeof(temp1),
+                "%s",
+                temp_ptr);
+        XtFree(temp_ptr);
+
+        (void)remove_leading_spaces(temp1);
+        (void)remove_trailing_spaces(temp1);
+
+        if(temp1[0] != '\0') {
+            // Yep, we have a custom path.  Warn the user that
+            // they're going to lose this path by destroying the
+            // dialog.
+            popup_message_always(langcode("POPEM00036"),
+                langcode("POPEM00040"));
+        }
+*/
+ 
+        XtPopdown(mw[ii].send_message_dialog);
+        XtDestroyWidget(mw[ii].send_message_dialog);
     }
 
-    mw[i].send_message_dialog = (Widget)NULL;
-    mw[i].to_call_sign[0] = '\0';
-    mw[i].send_message_call_data = (Widget)NULL;
-    mw[i].send_message_message_data = (Widget)NULL;
-    mw[i].send_message_text = (Widget)NULL;
+    mw[ii].send_message_dialog = (Widget)NULL;
+    mw[ii].to_call_sign[0] = '\0';
+    mw[ii].send_message_call_data = (Widget)NULL;
+    mw[ii].send_message_message_data = (Widget)NULL;
+    mw[ii].send_message_text = (Widget)NULL;
 
 end_critical_section(&send_message_dialog_lock, "messages_gui.c:Send_message_destroy_shell" );
 
@@ -306,7 +413,7 @@ begin_critical_section(&send_message_dialog_lock, "messages_gui.c:Check_new_call
             (void)to_upper(call_sign);
             get_path_data(call_sign, path, MAX_PATH);
         }
-        XmTextFieldSetString(mw[i].send_message_path, path);
+        XmTextFieldSetString(mw[i].send_message_reverse_path, path);
     }
 
 end_critical_section(&send_message_dialog_lock, "messages_gui.c:Check_new_call_messages" );
@@ -330,18 +437,19 @@ void Send_message_now( /*@unused@*/ Widget w, XtPointer clientData, /*@unused@*/
     char temp1[MAX_CALLSIGN+1];
     char temp2[302];
     char path[200];
-    int i;
+    int ii, jj;
     char *temp_ptr;
+    int substitution_made = 0;
 
 
-    i=atoi((char *)clientData);
+    ii=atoi((char *)clientData);
 
 begin_critical_section(&send_message_dialog_lock, "messages_gui.c:Send_message_now" );
 
-    if (mw[i].send_message_dialog) {
+    if (mw[ii].send_message_dialog) {
 
         /* find station and go there */
-        temp_ptr = XmTextFieldGetString(mw[i].send_message_call_data);
+        temp_ptr = XmTextFieldGetString(mw[ii].send_message_call_data);
         xastir_snprintf(temp1,
             sizeof(temp1),
             "%s",
@@ -351,16 +459,32 @@ begin_critical_section(&send_message_dialog_lock, "messages_gui.c:Send_message_n
         (void)remove_trailing_spaces(temp1);
         (void)to_upper(temp1);
 
-        temp_ptr = XmTextFieldGetString(mw[i].send_message_message_data);
+        temp_ptr = XmTextFieldGetString(mw[ii].send_message_message_data);
         xastir_snprintf(temp2,
             sizeof(temp2),
             "%s",
             temp_ptr);
         XtFree(temp_ptr);
 
+        // We have the message text now.  Check it for illegal
+        // characters, remove them and substitute spaces if found.
+        // Illegal characters are '|', '{', and '~' for messaging.
+        for (jj = 0; jj < strlen(temp2); jj++) {
+            if (       temp2[jj] == '|'
+                    || temp2[jj] == '{'
+                    || temp2[jj] == '~' ) {
+                temp2[jj] = '.';    // Replace with a dot
+                substitution_made++;
+            }
+        }
+        if (substitution_made) {
+            popup_message_always(langcode("POPEM00022"),
+                langcode("POPEM00039"));
+        }
+
         (void)remove_trailing_spaces(temp2);
 
-        temp_ptr = XmTextFieldGetString(mw[i].send_message_path);
+        temp_ptr = XmTextFieldGetString(mw[ii].send_message_path);
         xastir_snprintf(path,
             sizeof(path),
             "%s",
@@ -371,7 +495,11 @@ begin_critical_section(&send_message_dialog_lock, "messages_gui.c:Send_message_n
         (void)to_upper(path);
 
         if(debug_level & 2)
-            fprintf(stderr,"Send message to <%s> from <%s> :%s\n",temp1,mw[i].to_call_sign,temp2);
+            fprintf(stderr,
+                "Send message to <%s> from <%s> :%s\n",
+                temp1,
+                mw[ii].to_call_sign,
+                temp2);
 
         if ( (strlen(temp1) != 0)                       // Callsign field is not blank
                 && (strlen(temp2) != 0)                 // Message field is not blank
@@ -380,10 +508,10 @@ begin_critical_section(&send_message_dialog_lock, "messages_gui.c:Send_message_n
             auto_reply=0;
             XmToggleButtonSetState(auto_msg_toggle,FALSE,FALSE);
             statusline(langcode("BBARSTA011"),0);       // Auto Reply Messages OFF
-            output_message(mw[i].to_call_sign,temp1,temp2,path);
-            XmTextFieldSetString(mw[i].send_message_message_data,"");
-//            if (mw[i].message_group!=1)
-//                XtSetSensitive(mw[i].button_ok,FALSE);
+            output_message(mw[ii].to_call_sign,temp1,temp2,path);
+            XmTextFieldSetString(mw[ii].send_message_message_data,"");
+//            if (mw[ii].message_group!=1)
+//                XtSetSensitive(mw[ii].button_ok,FALSE);
         }
         else {
             // Could add a popup here someday that says something about
@@ -510,6 +638,25 @@ end_critical_section(&send_message_dialog_lock, "messages_gui.c:Clear_message_to
 
 
 
+void Kick_timer( /*@unused@*/ Widget w, XtPointer clientData, /*@unused@*/ XtPointer callData) {
+    char *temp_ptr;
+    char temp1[MAX_CALLSIGN+1];
+
+
+    temp_ptr = XmTextFieldGetString(mw[atoi(clientData)].send_message_call_data);
+        xastir_snprintf(temp1,
+            sizeof(temp1),
+            "%s",
+            temp_ptr);
+    XtFree(temp_ptr);
+
+    kick_outgoing_timer(temp1);
+}
+
+
+
+
+
 void Clear_messages_to( /*@unused@*/ Widget w, XtPointer clientData, /*@unused@*/ XtPointer callData) {
     char *temp_ptr;
     char temp1[MAX_CALLSIGN+1];
@@ -546,8 +693,11 @@ void Send_message_call( /*@unused@*/ Widget w, XtPointer clientData, /*@unused@*
 
 
 
+// The main Send Message dialog.  db.c:update_messages() is the
+// function which fills in the message history information.
+//
 void Send_message( /*@unused@*/ Widget w, XtPointer clientData, /*@unused@*/ XtPointer callData) {
-    Arg args[20];
+    Arg args[50];
     char temp[60];
     unsigned int n;
     int j,i;
@@ -557,6 +707,9 @@ void Send_message( /*@unused@*/ Widget w, XtPointer clientData, /*@unused@*/ XtP
     Atom delw;
 //    DataRow *p_station;
 
+
+//fprintf(stderr,"Send_message\n");
+
     groupon=0;
     box_len=90;
     i=0;
@@ -619,7 +772,7 @@ begin_critical_section(&send_message_dialog_lock, "messages_gui.c:Send_message"
         mw[i].form =  XtVaCreateWidget("Send_message form",
                 xmFormWidgetClass,
                 mw[i].pane,
-                XmNfractionBase, 4,
+                XmNfractionBase, 5,
                 XmNautoUnmanage, FALSE,
                 XmNshadowThickness, 1,
                 MY_FOREGROUND_COLOR,
@@ -648,7 +801,8 @@ begin_critical_section(&send_message_dialog_lock, "messages_gui.c:Send_message"
         XtSetArg(args[n], XmNautoShowCursorPosition, FALSE); n++;
         XtSetArg(args[n], XmNforeground, MY_FG_COLOR); n++;
 
-// This one causes segfaults, why?
+// This one causes segfaults, why?  Answer: args[] was set to 20
+// (too small)
 //        XtSetArg(args[n], XmNbackground, MY_BG_COLOR); n++;
  
         mw[i].send_message_text = XmCreateScrolledText(mw[i].form,
@@ -743,6 +897,30 @@ begin_critical_section(&send_message_dialog_lock, "messages_gui.c:Send_message"
                 XmNtraversalOn, TRUE,
                 NULL);
 
+        mw[i].send_message_reverse_path = XtVaCreateManagedWidget("Send_message reverse path", 
+                xmTextFieldWidgetClass, 
+                mw[i].form,
+                XmNeditable,   FALSE,
+                XmNcursorPositionVisible, FALSE,
+                XmNsensitive, TRUE,
+                XmNshadowThickness,    1,
+                XmNcolumns, 26,
+                XmNwidth, ((26*7)+2),
+                XmNmaxLength, 199,
+                XmNbackground, colors[0x0f],
+                XmNtopAttachment, XmATTACH_NONE,
+                XmNbottomAttachment, XmATTACH_FORM,
+                XmNbottomOffset, 80,
+                XmNleftAttachment, XmATTACH_WIDGET,
+                XmNleftWidget, mw[i].send_message_path,
+                XmNleftOffset, 5,
+                XmNrightAttachment,XmATTACH_NONE,
+                XmNnavigationType, XmTAB_GROUP,
+                XmNtraversalOn, FALSE,
+                MY_FOREGROUND_COLOR,
+                MY_BACKGROUND_COLOR,
+                NULL);
+
         mw[i].message = XtVaCreateManagedWidget(langcode("WPUPMSB008"),
                 xmLabelWidgetClass, 
                 mw[i].form,
@@ -810,6 +988,22 @@ begin_critical_section(&send_message_dialog_lock, "messages_gui.c:Send_message"
                 MY_BACKGROUND_COLOR,
                 NULL);
 
+        mw[i].button_kick_timer = XtVaCreateManagedWidget(langcode("WPUPMSB012"),
+                xmPushButtonGadgetClass, 
+                mw[i].form,
+                XmNleftAttachment, XmATTACH_POSITION,
+                XmNleftPosition, 2,
+                XmNrightAttachment, XmATTACH_POSITION,
+                XmNrightPosition, 3,
+                XmNtopAttachment, XmATTACH_NONE,
+                XmNbottomAttachment, XmATTACH_FORM,
+                XmNbottomOffset, 5,
+                XmNnavigationType, XmTAB_GROUP,
+                XmNtraversalOn, TRUE,
+                MY_FOREGROUND_COLOR,
+                MY_BACKGROUND_COLOR,
+                NULL);
+
         mw[i].button_ok = XtVaCreateManagedWidget(langcode("WPUPMSB009"),
                 xmPushButtonGadgetClass, 
                 mw[i].form,
@@ -817,9 +1011,9 @@ begin_critical_section(&send_message_dialog_lock, "messages_gui.c:Send_message"
                 XmNbottomAttachment, XmATTACH_FORM,
                 XmNbottomOffset, 5,
                 XmNleftAttachment, XmATTACH_POSITION,
-                XmNleftPosition, 2,
+                XmNleftPosition, 3,
                 XmNrightAttachment, XmATTACH_POSITION,
-                XmNrightPosition, 3,
+                XmNrightPosition, 4,
                 XmNnavigationType, XmTAB_GROUP,
                 XmNtraversalOn, TRUE,
                 MY_FOREGROUND_COLOR,
@@ -833,9 +1027,9 @@ begin_critical_section(&send_message_dialog_lock, "messages_gui.c:Send_message"
                 XmNbottomAttachment, XmATTACH_FORM,
                 XmNbottomOffset, 5,
                 XmNleftAttachment, XmATTACH_POSITION,
-                XmNleftPosition, 3,
+                XmNleftPosition, 4,
                 XmNrightAttachment, XmATTACH_POSITION,
-                XmNrightPosition, 4,
+                XmNrightPosition, 5,
                 XmNnavigationType, XmTAB_GROUP,
                 XmNtraversalOn, TRUE,
                 MY_FOREGROUND_COLOR,
@@ -855,6 +1049,8 @@ begin_critical_section(&send_message_dialog_lock, "messages_gui.c:Send_message"
 
         XtAddCallback(mw[i].button_clear_pending_msgs, XmNactivateCallback, Clear_messages_to, (XtPointer)mw[i].win);
 
+        XtAddCallback(mw[i].button_kick_timer, XmNactivateCallback, Kick_timer, (XtPointer)mw[i].win);
+
         XtAddCallback(mw[i].button_submit_call, XmNactivateCallback, Check_new_call_messages, (XtPointer)mw[i].call);
 
         if (clientData != NULL) {
@@ -862,7 +1058,7 @@ begin_critical_section(&send_message_dialog_lock, "messages_gui.c:Send_message"
 
             XmTextFieldSetString(mw[i].send_message_call_data, group);
             get_path_data(group, path, MAX_PATH);
-            XmTextFieldSetString(mw[i].send_message_path, path);
+            XmTextFieldSetString(mw[i].send_message_reverse_path, path);
         }
 
         pos_dialog(mw[i].send_message_dialog);
@@ -897,7 +1093,7 @@ end_critical_section(&send_message_dialog_lock, "messages_gui.c:Send_message" );
 /************************* Auto msg **************************************/
 /*************************************************************************/
 void Auto_msg_option( /*@unused@*/ Widget w, /*@unused@*/ XtPointer clientData, /*@unused@*/ XtPointer calldata) {
-    int item_no = (int) clientData;
+    int item_no = XTPOINTER_TO_INT(clientData);
 
     if (item_no)
         auto_reply = 1;
