@@ -1,9 +1,9 @@
 /*
- * $Id: track_gui.c,v 1.33 2004/08/19 06:38:21 we7u Exp $
+ * $Id: track_gui.c,v 1.39 2005/07/22 06:08:17 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2004  The Xastir Group
+ * Copyright (C) 2000-2005  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -148,13 +148,17 @@ void Track_station_now(Widget w, XtPointer clientData, XtPointer callData) {
         track_station_on = 1;   // Track it whether we've seen it yet or not
         if (!found) {
             xastir_snprintf(temp2, sizeof(temp2), langcode("POPEM00026"), temp);
-            popup_message(langcode("POPEM00025"),temp2);
+            popup_message_always(langcode("POPEM00025"),temp2);
+        }
+        if ( track_me & !is_my_call( tracking_station_call, 1) ) {
+            XmToggleButtonSetState( trackme_button, FALSE, FALSE );
+            track_me = 0;
         }
     } else {
         tracking_station_call[0] = '\0';    // Empty it out again
         track_station_on = 0;
         xastir_snprintf(temp2, sizeof(temp2), langcode("POPEM00002"), temp);
-        popup_message(langcode("POPEM00003"),temp2);
+        popup_message_always(langcode("POPEM00003"),temp2);
     }
 
     track_station_destroy_shell(w, clientData, callData);
@@ -395,16 +399,19 @@ end_critical_section(&download_findu_dialog_lock, "track_gui.c:Download_trail_de
 void Download_trail_now(Widget w, XtPointer clientData, XtPointer callData) {
     char temp[MAX_CALLSIGN+1];
     char fileimg[400];
-#ifdef HAVE_LIBCURL
-#else
-#ifdef HAVE_WGET
-    char tempfile[500];
-#endif  // HAVE_WGET
-#endif
     char log_filename[200];
     char *temp_ptr;
 
 
+    // Check whether it's ok to do a download currently.
+    if (read_file) {
+        // No, we're already in the middle of reading in some file.
+        // Skip trying to download yet another file.
+        popup_message_always(langcode("POPEM00035"),
+            langcode("POPEM00041"));
+        return;
+    }
+ 
     busy_cursor(appshell);
 
     xastir_snprintf(log_filename,
@@ -454,30 +461,11 @@ void Download_trail_now(Widget w, XtPointer clientData, XtPointer callData) {
         "http://www.findu.com/cgi-bin/rawposit.cgi?call=%s&start=%d&length=%d",         // Old
 //        "http://www.findu.com/cgi-bin/rawposit.cgi?call=%s&start=%d&length=%d&time=1",// New
         download_trail_station_call,posit_start,posit_length);
-#ifdef HAVE_LIBCURL
-    if (curl_getfile(fileimg, log_filename)) {
+
+    if (fetch_remote_file(fileimg, log_filename)) {
         // Had trouble getting the file.  Abort.
         return;
     }
-#else
-#ifdef HAVE_WGET
-    xastir_snprintf(tempfile, sizeof(tempfile),
-        "%s --server-response --timestamping --tries=1 --timeout=30 --output-document=%s \'%s\' 2> /dev/null\n",
-        WGET_PATH,
-        log_filename,
-        fileimg);
-
-    if (debug_level & 2)
-        fprintf(stderr,"%s",tempfile);
-
-    if ( system(tempfile) ) {   // Go get the file
-        fprintf(stderr,"Couldn't download the trail\n");
-        return;
-    }
-#else   // HAVE_WGET
-    fprintf(stderr,"libcurl or 'wget' not installed.  Can't download trail\n");
-#endif  // HAVE_WGET
-#endif
 
     // Set permissions on the file so that any user can overwrite it.
     chmod(log_filename, 0666);
@@ -512,6 +500,7 @@ void Reset_posit_length_max(Widget w, XtPointer clientData, XtPointer callData)
         XtVaSetValues(posit_length_value,
             XmNvalue, posit_start,
             NULL);
+        posit_length = temp;
     }
     XtVaSetValues(posit_length_value,
         XmNmaximum, posit_start,
