@@ -1,9 +1,9 @@
 /*
- * $Id: wx_gui.c,v 1.34 2004/08/21 06:48:00 we7u Exp $
+ * $Id: wx_gui.c,v 1.38 2005/07/31 06:19:14 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2004  The Xastir Group
+ * Copyright (C) 2000-2005  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -106,7 +106,7 @@ end_critical_section(&wx_detailed_alert_shell_lock, "wx_gui.c:wx_detailed_alert_
 void wx_alert_finger_output( Widget widget, char *handle) {
     static Widget pane, my_form, mess, button_cancel,wx_detailed_alert_list;
     Atom delw;
-    Arg al[20];                     // Arg List
+    Arg al[50];                     // Arg List
     register unsigned int ac = 0;   // Arg Count
     char temp[1024];
     XmString item;
@@ -314,13 +314,15 @@ end_critical_section(&wx_alert_shell_lock, "wx_gui.c:wx_alert_destroy_shell" );
 
 
 void wx_alert_update_list(void) {
-    int nn;         // index into alert table.  Starts at 0
+//    int nn;         // index into alert table.  Starts at 0
     int ii;         // index into dialog lines.  Starts at 1
     int max_item_count; // max dialog lines
     char temp[600];
     XmString item;
 
     if (wx_alert_shell) {
+        struct hashtable_itr *iterator;
+        alert_entry *alert;
 
 begin_critical_section(&wx_alert_shell_lock, "wx_gui.c:wx_alert_update_list" );
 
@@ -329,14 +331,22 @@ begin_critical_section(&wx_alert_shell_lock, "wx_gui.c:wx_alert_update_list" );
 
         ii = 0; // Current dialog count
 
-        // Step through the alert list.  Create a string for each
+        // Iterate through the alert hash.  Create a string for each
         // non-expired/non-blank entry.
-        for (nn = 0; nn < alert_max_count; nn++) {
+        iterator = create_wx_alert_iterator();
+        alert = get_next_wx_alert(iterator);
+        while (iterator != NULL && alert) {
             char status[10];
 
-            // Check whether alert slot is empty/filled
-            if (alert_list[nn].title[0] == '\0')    // It's empty
+            // Check whether alert record is empty/filled.  This
+            // code is from the earlier array implementation.  If
+            // we're expiring records from our hash properly we
+            // probably don't need this anymore.
+            //
+            if (alert->title[0] == '\0') {    // It's empty
+                alert = get_next_wx_alert(iterator);
                 continue;
+            }
 
             // AFGNPW      NWS-WARN    Until: 191500z   AK_Z213   WIND               P7IAA
             // TSATOR      NWS-ADVIS   Until: 190315z   OK_C127   TORNDO             H2VAA
@@ -349,28 +359,28 @@ begin_critical_section(&wx_alert_shell_lock, "wx_gui.c:wx_alert_update_list" );
             xastir_snprintf(temp,
                 sizeof(temp),
                 "%-9s%s   %-9s   %c%c @%c%c%c%cz ==> %c%c @%c%c%c%cz %s %-7s   %s   %s%s%s%s",
-                alert_list[nn].from,
-                alert_list[nn].seq,
-                alert_list[nn].to,
-                alert_list[nn].issue_date_time[0],
-                alert_list[nn].issue_date_time[1],
-                alert_list[nn].issue_date_time[2],
-                alert_list[nn].issue_date_time[3],
-                alert_list[nn].issue_date_time[4],
-                alert_list[nn].issue_date_time[5],
-                alert_list[nn].activity[0],
-                alert_list[nn].activity[1],
-                alert_list[nn].activity[2],
-                alert_list[nn].activity[3],
-                alert_list[nn].activity[4],
-                alert_list[nn].activity[5],
+                alert->from,
+                alert->seq,
+                alert->to,
+                alert->issue_date_time[0],
+                alert->issue_date_time[1],
+                alert->issue_date_time[2],
+                alert->issue_date_time[3],
+                alert->issue_date_time[4],
+                alert->issue_date_time[5],
+                alert->activity[0],
+                alert->activity[1],
+                alert->activity[2],
+                alert->activity[3],
+                alert->activity[4],
+                alert->activity[5],
                 status,
-                alert_list[nn].title,
-                alert_list[nn].alert_tag,
-                alert_list[nn].desc0,
-                alert_list[nn].desc1,
-                alert_list[nn].desc2,
-                alert_list[nn].desc3);
+                alert->title,
+                alert->alert_tag,
+                alert->desc0,
+                alert->desc1,
+                alert->desc2,
+                alert->desc3);
 
 
             item = XmStringCreateLtoR(temp, XmFONTLIST_DEFAULT_TAG);
@@ -390,7 +400,11 @@ begin_critical_section(&wx_alert_shell_lock, "wx_gui.c:wx_alert_update_list" );
 
             XmStringFree(item);
 
+            alert = get_next_wx_alert(iterator);
+
         }   // End of for loop
+        free(iterator);
+
 
         // If we have fewer alerts now, delete the extras from the window
         if (ii < max_item_count) {
@@ -409,7 +423,7 @@ end_critical_section(&wx_alert_shell_lock, "wx_gui.c:wx_alert_update_list" );
 void Display_Wx_Alert( /*@unused@*/ Widget wdgt, /*@unused@*/ XtPointer clientData, /*@unused@*/ XtPointer callData) {
     static Widget pane, my_form, mess, button_cancel;
     Atom delw;
-    Arg al[20];                    /* Arg List */
+    Arg al[50];                    /* Arg List */
     register unsigned int ac = 0;           /* Arg Count */
 
     if(!wx_alert_shell) {
@@ -1502,7 +1516,7 @@ begin_critical_section(&wx_station_dialog_lock, "wx_gui.c:fill_wx_data" );
 
                 if (weather != 0) {  // we have weather data
                     if (strlen(weather->wx_temp) > 0) {
-                       if (!units_english_metric) {
+                       if (!english_units) {
                             xastir_snprintf(temp,
                                 sizeof(temp),
                                 "%03d",
@@ -1522,7 +1536,7 @@ begin_critical_section(&wx_station_dialog_lock, "wx_gui.c:fill_wx_data" );
                     XtManageChild(WX_wind_cse_data);
 
                     if (strlen(weather->wx_speed) > 0) {
-                        if (!units_english_metric) {
+                        if (!english_units) {
                             xastir_snprintf(temp,
                                 sizeof(temp),
                                 "%03d",
@@ -1537,7 +1551,7 @@ begin_critical_section(&wx_station_dialog_lock, "wx_gui.c:fill_wx_data" );
                     XtManageChild(WX_wind_spd_data);
 
                     if (strlen(weather->wx_gust) > 0) {
-                        if (!units_english_metric) {
+                        if (!english_units) {
                             xastir_snprintf(temp,
                                 sizeof(temp),
                                 "%03d",
@@ -1552,7 +1566,7 @@ begin_critical_section(&wx_station_dialog_lock, "wx_gui.c:fill_wx_data" );
                     XtManageChild(WX_wind_gst_data);
 
                     if (strlen(weather->wx_rain_total) > 0) {
-                        if (!units_english_metric)
+                        if (!english_units)
                             xastir_snprintf(temp,
                                 sizeof(temp),
                                 "%0.2f",
@@ -1570,7 +1584,7 @@ begin_critical_section(&wx_station_dialog_lock, "wx_gui.c:fill_wx_data" );
                     XtManageChild(WX_rain_data);
 
                     if (strlen(weather->wx_rain) > 0) {
-                        if (!units_english_metric)
+                        if (!english_units)
                             xastir_snprintf(temp,
                                 sizeof(temp),
                                 "%0.2f",
@@ -1588,7 +1602,7 @@ begin_critical_section(&wx_station_dialog_lock, "wx_gui.c:fill_wx_data" );
                     XtManageChild(WX_rain_h_data);
 
                     if (strlen(weather->wx_prec_24) > 0) {
-                        if (!units_english_metric)
+                        if (!english_units)
                             xastir_snprintf(temp,
                                 sizeof(temp),
                                 "%0.2f",
@@ -1606,7 +1620,7 @@ begin_critical_section(&wx_station_dialog_lock, "wx_gui.c:fill_wx_data" );
                     XtManageChild(WX_rain_24_data);
 
                     if (strlen(weather->wx_prec_00) > 0) {
-                        if (!units_english_metric)
+                        if (!english_units)
                             xastir_snprintf(temp,
                                 sizeof(temp),
                                 "%0.2f",
@@ -1631,7 +1645,7 @@ begin_critical_section(&wx_station_dialog_lock, "wx_gui.c:fill_wx_data" );
                     XtManageChild(WX_humidity_data);
 
                     if (strlen(wx_dew_point) > 0) {
-                        if (!units_english_metric) {
+                        if (!english_units) {
                             xastir_snprintf(temp,
                                 sizeof(temp),
                                 "%03d",
@@ -1645,7 +1659,7 @@ begin_critical_section(&wx_station_dialog_lock, "wx_gui.c:fill_wx_data" );
                     XtManageChild(WX_dew_point_data);
 
                     if (strlen(wx_high_wind) > 0) {
-                        if (!units_english_metric) {
+                        if (!english_units) {
                             xastir_snprintf(temp,
                                 sizeof(temp),
                                 "%03d",
@@ -1659,7 +1673,7 @@ begin_critical_section(&wx_station_dialog_lock, "wx_gui.c:fill_wx_data" );
                     XtManageChild(WX_high_wind_data);
 
                     if (strlen(wx_wind_chill) > 0) {
-                        if (!units_english_metric) {
+                        if (!english_units) {
                             xastir_snprintf(temp,
                                 sizeof(temp),
                                 "%03d",
@@ -1673,7 +1687,7 @@ begin_critical_section(&wx_station_dialog_lock, "wx_gui.c:fill_wx_data" );
                     XtManageChild(WX_wind_chill_data);
 
                     if (strlen(weather->wx_baro) > 0) {
-                        if (!units_english_metric) {
+                        if (!english_units) {
                             //xastir_snprintf(temp, sizeof(temp), "%0.0f",
                             //        atof(wx_baro_inHg)*25.4); // inch Hg -> mm Hg
                             //XmTextFieldSetString(WX_baro_data,temp);
@@ -1692,7 +1706,7 @@ begin_critical_section(&wx_station_dialog_lock, "wx_gui.c:fill_wx_data" );
                     XtManageChild(WX_baro_data);
 
                     if (wx_three_hour_baro_on) {
-                        if (!units_english_metric) {  // hPa
+                        if (!english_units) {  // hPa
                             //xastir_snprintf(temp, sizeof(temp), "%0.0f",
                             //        atof(wx_three_hour_baro)*25.4); // inch Hg -> mm Hg
                             XmTextFieldSetString(WX_three_hour_baro_data,wx_three_hour_baro);
@@ -1709,7 +1723,7 @@ begin_critical_section(&wx_station_dialog_lock, "wx_gui.c:fill_wx_data" );
                     XtManageChild(WX_three_hour_baro_data);
 
                     if (wx_hi_temp_on) {
-                        if (!units_english_metric) {
+                        if (!english_units) {
                             xastir_snprintf(temp,
                                 sizeof(temp),
                                 "%03d",
@@ -1723,7 +1737,7 @@ begin_critical_section(&wx_station_dialog_lock, "wx_gui.c:fill_wx_data" );
                     XtManageChild(WX_hi_temp_data);
 
                     if (wx_low_temp_on) {
-                        if (!units_english_metric) {
+                        if (!english_units) {
                             xastir_snprintf(temp,
                                 sizeof(temp),
                                 "%03d",
@@ -1737,7 +1751,7 @@ begin_critical_section(&wx_station_dialog_lock, "wx_gui.c:fill_wx_data" );
                     XtManageChild(WX_low_temp_data);
 
                     if (wx_heat_index_on) {
-                        if (!units_english_metric) {
+                        if (!english_units) {
                             xastir_snprintf(temp,
                                 sizeof(temp),
                                 "%03d",
@@ -1754,105 +1768,105 @@ begin_critical_section(&wx_station_dialog_lock, "wx_gui.c:fill_wx_data" );
         }
 
         /* labels */
-        if (!units_english_metric)
+        if (!english_units)
             XmTextFieldSetString(WX_speed_label,langcode("UNIOP00012"));
         else
             XmTextFieldSetString(WX_speed_label,langcode("UNIOP00013"));
 
         XtManageChild(WX_speed_label);
 
-        if (!units_english_metric)
+        if (!english_units)
             XmTextFieldSetString(WX_gust_label,langcode("UNIOP00012"));
         else
             XmTextFieldSetString(WX_gust_label,langcode("UNIOP00013"));
 
         XtManageChild(WX_gust_label);
 
-        if (!units_english_metric)
+        if (!english_units)
             XmTextFieldSetString(WX_temp_label,langcode("UNIOP00014"));
         else
             XmTextFieldSetString(WX_temp_label,langcode("UNIOP00015"));
 
         XtManageChild(WX_temp_label);
 
-        if (!units_english_metric)
+        if (!english_units)
             XmTextFieldSetString(WX_rain_label,langcode("UNIOP00016"));
         else
             XmTextFieldSetString(WX_rain_label,langcode("UNIOP00017"));
 
         XtManageChild(WX_rain_label);
 
-        if (!units_english_metric)
+        if (!english_units)
             XmTextFieldSetString(WX_to_rain_label,langcode("UNIOP00022"));
         else
             XmTextFieldSetString(WX_to_rain_label,langcode("UNIOP00023"));
 
         XtManageChild(WX_to_rain_label);
 
-        if (!units_english_metric)
+        if (!english_units)
             XmTextFieldSetString(WX_rain_h_label,langcode("UNIOP00020"));
         else
             XmTextFieldSetString(WX_rain_h_label,langcode("UNIOP00021"));
 
         XtManageChild(WX_rain_h_label);
 
-        if (!units_english_metric)
+        if (!english_units)
             XmTextFieldSetString(WX_rain_24_label,langcode("UNIOP00018"));
         else
             XmTextFieldSetString(WX_rain_24_label,langcode("UNIOP00019"));
 
         XtManageChild(WX_rain_24_label);
 
-        if (!units_english_metric)
+        if (!english_units)
             XmTextFieldSetString(WX_dew_point_label,langcode("UNIOP00014"));
         else
             XmTextFieldSetString(WX_dew_point_label,langcode("UNIOP00015"));
 
         XtManageChild(WX_dew_point_label);
 
-        if (!units_english_metric)
+        if (!english_units)
             XmTextFieldSetString(WX_wind_chill_label,langcode("UNIOP00014"));
         else
             XmTextFieldSetString(WX_wind_chill_label,langcode("UNIOP00015"));
 
         XtManageChild(WX_wind_chill_label);
 
-        if (!units_english_metric)
+        if (!english_units)
             XmTextFieldSetString(WX_heat_index_label,langcode("UNIOP00014"));
         else
             XmTextFieldSetString(WX_heat_index_label,langcode("UNIOP00015"));
 
         XtManageChild(WX_heat_index_label);
 
-        if (!units_english_metric)
+        if (!english_units)
             XmTextFieldSetString(WX_hi_temp_label,langcode("UNIOP00014"));
         else
             XmTextFieldSetString(WX_hi_temp_label,langcode("UNIOP00015"));
 
         XtManageChild(WX_hi_temp_label);
 
-        if (!units_english_metric)
+        if (!english_units)
             XmTextFieldSetString(WX_low_temp_label,langcode("UNIOP00014"));
         else
             XmTextFieldSetString(WX_low_temp_label,langcode("UNIOP00015"));
 
         XtManageChild(WX_low_temp_label);
 
-        if (!units_english_metric)
+        if (!english_units)
             XmTextFieldSetString(WX_high_wind_label,langcode("UNIOP00012"));
         else
             XmTextFieldSetString(WX_high_wind_label,langcode("UNIOP00013"));
 
         XtManageChild(WX_high_wind_label);
 
-        if (!units_english_metric)
+        if (!english_units)
             XmTextFieldSetString(WX_baro_label,langcode("UNIOP00025"));
         else
             XmTextFieldSetString(WX_baro_label,langcode("UNIOP00027"));
 
         XtManageChild(WX_baro_label);
 
-        if (!units_english_metric)
+        if (!english_units)
             XmTextFieldSetString(WX_three_hour_baro_label,langcode("UNIOP00025"));
         else
             XmTextFieldSetString(WX_three_hour_baro_label,langcode("UNIOP00027"));
