@@ -1,8 +1,8 @@
 /*
- * $Id: x_spider.c,v 1.24 2004/08/21 06:35:23 we7u Exp $
+ * $Id: x_spider.c,v 1.36 2005/07/17 06:17:56 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
- * Copyright (C) 2003-2004  The Xastir Group
+ * Copyright (C) 2003-2005  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -167,9 +167,18 @@
 #endif  // SIGRET
 
 
-// This is from util.h/util.c.  We can't include util.h here because
-// it causes other problems.
+// Define this if you wish to use this as a standalone program
+// instead of as a function in another program.
+//
+//#define STANDALONE_PROGRAM
+
+
+
+// These are from util.h/util.c.  We can't include util.h here
+// because it causes other problems.
 extern short checkHash(char *theCall, short theHash);
+extern void get_timestamp(char *timestring);
+
 
 
 typedef struct _pipe_object {
@@ -351,6 +360,8 @@ void str_echo2(int sockfd, int pipe_from_parent, int pipe_to_parent) {
         //
         // Read data from socket, write to pipe (parent)
         //
+        if (!sockfd)    // Socket is closed
+            return; // Connection terminated
  
         n = readline(sockfd, line, MAXLINE);
         if (n == 0) {
@@ -363,13 +374,17 @@ void str_echo2(int sockfd, int pipe_from_parent, int pipe_to_parent) {
                 //fprintf(stderr,"EAGAIN or EWOULDBLOCK\n");
             }
             else {  // Non-normal error.  Report it.
-                fprintf(stderr,"str_echo2: Readline error: %d\n",errno);
+                fprintf(stderr,"str_echo2: Readline error socket: %d\n",errno);
+//close(sockfd);
+return;
             }
         }
         else {  // We received some data.  Send it down the pipe.
 //            fprintf(stderr,"str_echo2: %s\n",line);
             if (writen(pipe_to_parent, line, n) != n) {
-                fprintf(stderr,"str_echo2: Writen error: %d\n",errno);
+                fprintf(stderr,"str_echo2: Writen error socket: %d\n",errno);
+//close(sockfd);
+return;
             }
         }
 
@@ -377,6 +392,8 @@ void str_echo2(int sockfd, int pipe_from_parent, int pipe_to_parent) {
         //
         // Read data from pipe (parent), write to socket
         //
+        if (!pipe_from_parent)
+            exit(0);
  
         n = readline(pipe_from_parent, line, MAXLINE);
         if (n == 0) {
@@ -389,13 +406,18 @@ void str_echo2(int sockfd, int pipe_from_parent, int pipe_to_parent) {
                 //fprintf(stderr,"EAGAIN or EWOULDBLOCK\n");
             }
             else {  // Non-normal error.  Report it.
-                fprintf(stderr,"str_echo2: Readline error: %d\n",errno);
+                fprintf(stderr,"str_echo2: Readline error pipe: %d\n",errno);
+//close(pipe_from_parent);
+return;
             }
         }
         else {  // We received some data.  Send it down the socket.
 //            fprintf(stderr,"str_echo2: %s\n",line);
+
             if (writen(sockfd, line, n) != n) {
-                fprintf(stderr,"str_echo2: Writen error: %d\n",errno);
+                fprintf(stderr,"str_echo2: Writen error pipe: %d\n",errno);
+//close(pipe_from_parent);
+return;
             }
         }
 
@@ -459,17 +481,22 @@ int pipe_check(char *client_address) {
         //
         n = readline(p->to_parent[0], line, MAXLINE);
         if (n == 0) {
+            char timestring[101];
             pipe_object *q = pipe_head;
 
+            get_timestamp(timestring);
+
             if (p->authenticated) { 
                 fprintf(stderr,
-                    "X_spider session terminated, callsign: %s, address: %s\n",
+                    "%s X_spider session terminated, callsign: %s, address: %s\n",
+                    timestring,
                     p->callsign,
                     client_address);
             }
             else {
                 fprintf(stderr,
-                    "X_spider session terminated, unauthenticated user, address %s\n",
+                    "%s X_spider session terminated, unauthenticated user, address %s\n",
+                    timestring,
                     client_address);
             }
 
@@ -488,7 +515,8 @@ int pipe_check(char *client_address) {
                     q->next = p->next;
             }
             free(p);    // Free the malloc'd memory.
-            wait(0);    // Reap the status of the dead process
+
+            wait(NULL); // Reap the status of the dead process
         }
         else if (n < 0) {
             //fprintf(stderr,"pipe_check: Readline error: %d\n",errno);
@@ -520,6 +548,10 @@ int pipe_check(char *client_address) {
                 short passcode;
                 char *space;
 
+                // We have a string with user/pass in it, but they
+                // can be anywhere along the line.  We'll have to
+                // search for each piece.
+
 //fprintf(stderr,"x_spider:Found an authentication string\n");
 
                 // Copy the line
@@ -530,12 +562,29 @@ int pipe_check(char *client_address) {
                     "                                    ",
                     sizeof(line2) - strlen(line2));
 
-                // Snag the callsign string
+                // Find the "user" string position
+                callsign = strstr(line2,"user");
 
-                callsign = &line2[5];
+                if (callsign == NULL)
+                    continue;
+
+                // Fast-forward past the "user" word.
+                callsign += 4;
+
+                // Skip past any additional spaces that might be
+                // present between "user" and callsign.
+                while (callsign[0] == ' ' && callsign[0] != '\0') {
+                    callsign += 1;
+                }
+
+                if (callsign[0] == '\0')
+                    continue;
+
+                // We should now be pointing at the beginning of the
+                // callsign.
 
                 // Find the space after the callsign
-                space = strstr(&line2[5]," ");
+                space = strstr(callsign," ");
 
                 if (space == NULL)
                     continue;
@@ -545,14 +594,23 @@ int pipe_check(char *client_address) {
 
                 // Snag the passcode string 
 
-                // Find the "pass" string first
+                // Find the "pass" string
                 passcode_str = strstr(&space[1],"pass");
 
                 if (passcode_str == NULL)
                     continue;
 
-                // This is the start of the string we want
-                passcode_str = passcode_str + 5;
+                // Fast-forward past the "pass" word.
+                passcode_str = passcode_str + 4;
+
+                // Skip past any additional spaces that might be
+                // present between "pass" and the passcode.
+                while (passcode_str[0] == ' ' && passcode_str[0] != '\0') {
+                    passcode_str += 1;
+                }
+
+                if (passcode_str[0] == '\0')
+                    continue;
 
                 // Find the space after the passcode
                 space = strstr(&passcode_str[0]," ");
@@ -585,6 +643,9 @@ int pipe_check(char *client_address) {
                         "X_spider: Bad authentication, user %s, pass %d\n",
                         callsign,
                         passcode);
+                    fprintf(stderr,
+                        "Line: %s\n",
+                        line);
                 }
             }
 
@@ -659,6 +720,22 @@ int pipe_check(char *client_address) {
 // Also send it down the socket.
         pipe_object *q = pipe_head;
 
+        // The internet protocol for sending lines is "\r\n".  Knock
+        // off any line-end characters that might be present, then
+        // add a "\r\n" combo on the end.
+        //
+        if (n > 0 && (line[n-1] == '\r' || line[n-1] == '\n')) {
+            line[n-1] = '\0';
+            n--;
+        }
+        if (n > 0 && (line[n-1] == '\r' || line[n-1] == '\n')) {
+            line[n-1] = '\0';
+            n--;
+        }
+        // Add carriage-return/linefeed onto the end
+        strncat(line, "\r\n", 2);
+        n = n+2;
+
         while (q != NULL) {
 //          fprintf(stderr,"pipe_check: %s\n",line);
 
@@ -739,18 +816,18 @@ void set_proc_title(char *fmt,...) {
     char *p;
     int i,maxlen = (LastArgv - Argv[0]) - 2;
 
-    //printf("DEBUG: maxlen: %i\n", maxlen);
+    //fprintf(stderr,"DEBUG: maxlen: %i\n", maxlen);
 
     va_start(msg,fmt);
 
     memset(statbuf, 0, sizeof(statbuf));
-    vsnprintf(statbuf, sizeof(statbuf), fmt, msg);
+    xastir_vsnprintf(statbuf, sizeof(statbuf), fmt, msg);
 
     va_end(msg);
 
     i = strlen(statbuf);
 
-    snprintf(Argv[0], maxlen, "%s", statbuf);
+    xastir_snprintf(Argv[0], maxlen, "%s", statbuf);
     p = &Argv[0][i];
   
     while(p < LastArgv)
@@ -788,14 +865,20 @@ void set_proc_title(char *fmt,...) {
 // socket and exit from the child process.  Notify the main process
 // as well?
 //
-//int main(int argc, char *argv[]) {
+#ifdef STANDALONE_PROGRAM
+int main(int argc, char *argv[]) {
+#else   // !STANDALONE_PROGRAM 
 void Server(int argc, char *argv[], char *envp[]) {
-    int sockfd, newsockfd, clilen, childpid;
+#endif  // STANDALONE_PROGRAM
+
+    int sockfd, newsockfd, childpid;
+    socklen_t clilen;
     struct sockaddr_in cli_addr, serv_addr;
     pipe_object *p;
     int sendbuff;
     int pipe_to_parent; /* symbolic names to reduce confusion */
     int pipe_from_parent;
+    char timestring[101];
 
     
     // Open a TCP listening socket
@@ -853,7 +936,7 @@ void Server(int argc, char *argv[], char *envp[]) {
         // concurrent server (allows multiple concurrent
         // connections).
 
-        clilen = sizeof(cli_addr);
+        clilen = (socklen_t)sizeof(cli_addr);
 
         // "accept" is the call where we wait for a connection.  We
         // made the socket non-blocking above so that we pop out of
@@ -906,7 +989,10 @@ void Server(int argc, char *argv[], char *envp[]) {
         p->authenticated = 0;
         p->callsign[0] = '\0';
 
-        fprintf(stderr,"X_spider client connected from address %s\n",
+        get_timestamp(timestring);
+
+        fprintf(stderr,"%s X_spider client connected from address %s\n",
+            timestring,
             inet_ntoa(cli_addr.sin_addr));
  
         // Link it into the head of the chain.
@@ -965,7 +1051,7 @@ void Server(int argc, char *argv[], char *envp[]) {
                 "x-spider client @",
                 inet_ntoa(cli_addr.sin_addr),
                 "(xastir)");
-            //printf("DEBUG: %s\n", Argv[0]);
+            //fprintf(stderr,"DEBUG: %s\n", Argv[0]);
 #endif  // __linux__
 
 // It'd be very cool here to include the IP address of the remote
@@ -1037,6 +1123,7 @@ finis:
 // integrated with Xastir, instead of having to have a socket to
 // communicate between Xastir and the server.
 //
+#ifndef STANDALONE_PROGRAM
 int Fork_server(int argc, char *argv[], char *envp[]) {
     int childpid;
 
@@ -1082,7 +1169,7 @@ int Fork_server(int argc, char *argv[], char *envp[]) {
 #ifdef __linux__
         init_set_proc_title(argc, argv, envp);
         set_proc_title("%s", "x-spider daemon (xastir)");
-        //printf("DEBUG: %s\n", Argv[0]);
+        //fprintf(stderr,"DEBUG: %s\n", Argv[0]);
 #endif  // __linux__
  
 
@@ -1140,5 +1227,6 @@ int Fork_server(int argc, char *argv[], char *envp[]) {
     // when Xastir quits or segfaults.
     return(childpid);   // Really the parent PID in this case
 }
+#endif  // STANDALONE_PROGRAM
 
 
