@@ -1,9 +1,9 @@
 /* -*- c-basic-offset: 4; indent-tabs-mode: nil -*-
- * $Id: xa_config.c,v 1.125 2004/09/16 14:47:17 we7u Exp $
+ * $Id: xa_config.c,v 1.138 2005/07/29 19:29:59 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2004  The Xastir Group
+ * Copyright (C) 2000-2005  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -444,6 +444,8 @@ void save_data(void)  {
         store_long (fout, "SCREEN_LAT", mid_y_lat_offset);
         store_long (fout, "SCREEN_LONG", mid_x_long_offset);
 
+        store_string(fout, "RELAY_DIGIPEAT_CALLS", relay_digipeater_calls);
+
         store_int (fout, "COORDINATE_SYSTEM", coordinate_system);
 
         store_int (fout, "STATION_TRANSMIT_AMB", position_amb_chars);
@@ -500,7 +502,7 @@ void save_data(void)  {
         store_string (fout, "MAPS_LABEL_FONT", rotated_label_fontname[FONT_DEFAULT]);
 //N0VH
 #if defined(HAVE_IMAGEMAGICK)
-        store_int (fout, "TIGERMAP_TIMEOUT", tigermap_timeout);
+        store_int (fout, "NET_MAP_TIMEOUT", net_map_timeout);
         store_int (fout, "TIGERMAP_SHOW_GRID", tiger_show_grid);
         store_int (fout, "TIGERMAP_SHOW_COUNTIES", tiger_show_counties);
         store_int (fout, "TIGERMAP_SHOW_CITIES", tiger_show_cities);
@@ -517,6 +519,23 @@ void save_data(void)  {
         store_int (fout, "TIGERMAP_SHOW_MISC", tiger_show_misc);
 #endif //HAVE_IMAGEMAGICK
 
+#ifdef HAVE_LIBGEOTIFF
+        store_int (fout, "DRG_XOR_COLORS", DRG_XOR_colors);
+        store_int (fout, "DRG_SHOW_COLORS_0", DRG_show_colors[0]);
+        store_int (fout, "DRG_SHOW_COLORS_1", DRG_show_colors[1]);
+        store_int (fout, "DRG_SHOW_COLORS_2", DRG_show_colors[2]);
+        store_int (fout, "DRG_SHOW_COLORS_3", DRG_show_colors[3]);
+        store_int (fout, "DRG_SHOW_COLORS_4", DRG_show_colors[4]);
+        store_int (fout, "DRG_SHOW_COLORS_5", DRG_show_colors[5]);
+        store_int (fout, "DRG_SHOW_COLORS_6", DRG_show_colors[6]);
+        store_int (fout, "DRG_SHOW_COLORS_7", DRG_show_colors[7]);
+        store_int (fout, "DRG_SHOW_COLORS_8", DRG_show_colors[8]);
+        store_int (fout, "DRG_SHOW_COLORS_9", DRG_show_colors[9]);
+        store_int (fout, "DRG_SHOW_COLORS_10", DRG_show_colors[10]);
+        store_int (fout, "DRG_SHOW_COLORS_11", DRG_show_colors[11]);
+        store_int (fout, "DRG_SHOW_COLORS_12", DRG_show_colors[12]);
+#endif  // HAVE_LIBGEOTIFF
+
         // filter values
         // NOT SAVED: Select_.none
         store_int (fout, "DISPLAY_MY_STATION",            Select_.mine);
@@ -563,7 +582,7 @@ void save_data(void)  {
         store_int (fout, "DISPLAY_DEAD_RECKONING_SYMBOL", Display_.dr_symbol);
 
 
-        store_int (fout, "DISPLAY_UNITS_ENGLISH",         units_english_metric);
+        store_int (fout, "DISPLAY_UNITS_ENGLISH",         english_units);
         store_int (fout, "DISPLAY_DIST_BEAR_STATUS",      do_dbstatus);
 
 
@@ -703,6 +722,13 @@ void save_data(void)  {
             strncat (name, "TNC_FULLDUPLEX", sizeof(name) - strlen(name));
             store_int (fout, name, devices[i].fullduplex);
 
+            xastir_snprintf(name,
+                sizeof(name),
+                "%s",
+                name_temp);
+            strncat (name, "TNC_INIT_KISSMODE", sizeof(name) - strlen(name));
+            store_int (fout, name, devices[i].init_kiss);
+
             xastir_snprintf(name,
                 sizeof(name),
                 "%s",
@@ -1038,6 +1064,15 @@ void load_data_or_default(void) {
     if (!get_long ("SCREEN_LONG", &mid_x_long_offset, 0l, 129600000l, 64800000l))
         mid_x_long_offset = 64800000l;
 
+
+    if (!get_string("RELAY_DIGIPEAT_CALLS", relay_digipeater_calls, sizeof(relay_digipeater_calls)))
+        sprintf (relay_digipeater_calls, "WIDE1-1");
+    // Make them all upper-case.
+    (void)to_upper(relay_digipeater_calls);
+    // And take out all spaces
+    (void)remove_all_spaces(relay_digipeater_calls);
+
+
     if (!get_int ("COORDINATE_SYSTEM", &coordinate_system, 0, 5, USE_DDMMMM))
         coordinate_system = USE_DDMMMM;
 
@@ -1056,12 +1091,16 @@ void load_data_or_default(void) {
     if (!get_string("IMAGEMAGICK_GAMMA_ADJUST", name, sizeof(name)))
         imagemagick_gamma_adjust = 0.0;
     else
-        sscanf(name, "%f", &imagemagick_gamma_adjust);
+        if (1 != sscanf(name, "%f", &imagemagick_gamma_adjust)) {
+            fprintf(stderr,"load_data_or_default:sscanf parsing error\n");
+        }
 #endif  // HAVE_IMAGEMAGICK
     if (!get_string("RASTER_MAP_INTENSITY", name, sizeof(name)))
         raster_map_intensity = 1.0;
     else
-        sscanf(name, "%f", &raster_map_intensity);
+        if (1 != sscanf(name, "%f", &raster_map_intensity)) {
+            fprintf(stderr,"load_data_or_default:sscanf parsing error\n");
+        }
 #endif  // NO_GRAPHICS
 
     if (!get_int ("MAP_LETTERSTYLE", &letter_style, 0, 2, 1))
@@ -1169,37 +1208,37 @@ void load_data_or_default(void) {
     if (!get_string ("MAPS_LABEL_FONT_TINY", rotated_label_fontname[FONT_TINY], sizeof(rotated_label_fontname[FONT_TINY])))
         xastir_snprintf(rotated_label_fontname[FONT_TINY],
             sizeof(rotated_label_fontname[FONT_TINY]),
-            "-adobe-helvetica-medium-o-normal--8-*-*-*-*-*-iso8859-1");
+            "-adobe-helvetica-medium-r-normal--8-*-*-*-*-*-iso8859-1");
 
     if (!get_string ("MAPS_LABEL_FONT_SMALL", rotated_label_fontname[FONT_SMALL], sizeof(rotated_label_fontname[FONT_SMALL])))
         xastir_snprintf(rotated_label_fontname[FONT_SMALL],
             sizeof(rotated_label_fontname[FONT_SMALL]),
-            "-adobe-helvetica-medium-o-normal--10-*-*-*-*-*-iso8859-1");
+            "-adobe-helvetica-medium-r-normal--10-*-*-*-*-*-iso8859-1");
 
     if (!get_string ("MAPS_LABEL_FONT_MEDIUM", rotated_label_fontname[FONT_MEDIUM], sizeof(rotated_label_fontname[FONT_MEDIUM])))
         xastir_snprintf(rotated_label_fontname[FONT_MEDIUM],
             sizeof(rotated_label_fontname[FONT_MEDIUM]),
-            "-adobe-helvetica-medium-o-normal--12-*-*-*-*-*-iso8859-1");
+            "-adobe-helvetica-medium-r-normal--12-*-*-*-*-*-iso8859-1");
 
     if (!get_string ("MAPS_LABEL_FONT_LARGE", rotated_label_fontname[FONT_LARGE], sizeof(rotated_label_fontname[FONT_LARGE])))
         xastir_snprintf(rotated_label_fontname[FONT_LARGE],
             sizeof(rotated_label_fontname[FONT_LARGE]),
-            "-adobe-helvetica-medium-o-normal--14-*-*-*-*-*-iso8859-1");
+            "-adobe-helvetica-medium-r-normal--14-*-*-*-*-*-iso8859-1");
 
     if (!get_string ("MAPS_LABEL_FONT_HUGE", rotated_label_fontname[FONT_HUGE], sizeof(rotated_label_fontname[FONT_HUGE])))
         xastir_snprintf(rotated_label_fontname[FONT_HUGE],
             sizeof(rotated_label_fontname[FONT_HUGE]),
-            "-adobe-helvetica-medium-o-normal--24-*-*-*-*-*-iso8859-1");
+            "-adobe-helvetica-medium-r-normal--24-*-*-*-*-*-iso8859-1");
 
     if (!get_string ("MAPS_LABEL_FONT", rotated_label_fontname[FONT_DEFAULT], sizeof(rotated_label_fontname[FONT_DEFAULT])))
         xastir_snprintf(rotated_label_fontname[FONT_DEFAULT],
             sizeof(rotated_label_fontname[FONT_DEFAULT]),
-            "-adobe-helvetica-medium-o-normal--12-*-*-*-*-*-iso8859-1");
+            "-adobe-helvetica-medium-r-normal--12-*-*-*-*-*-iso8859-1");
 
 //N0VH
 #if defined(HAVE_IMAGEMAGICK)
-    if (!get_int ("TIGERMAP_TIMEOUT", &tigermap_timeout, 10, 180, 30))
-        tigermap_timeout = 30;
+    if (!get_int ("NET_MAP_TIMEOUT", &net_map_timeout, 10, 300, 120))
+        net_map_timeout = 90;
 
     if (!get_int ("TIGERMAP_SHOW_GRID", &tiger_show_grid, 0, 1, 0))
         tiger_show_grid = 0;
@@ -1242,9 +1281,52 @@ void load_data_or_default(void) {
 
     if (!get_int ("TIGERMAP_SHOW_MISC", &tiger_show_misc, 0, 1, 1))
         tiger_show_misc = 1;
-
 #endif //HAVE_IMAGEMAGICK
 
+#ifdef HAVE_LIBGEOTIFF
+    if (!get_int ("DRG_XOR_COLORS", &DRG_XOR_colors, 0, 1, 0))
+        DRG_XOR_colors = 0;
+
+    if (!get_int ("DRG_SHOW_COLORS_0", &DRG_show_colors[0], 0, 1, 1))
+        DRG_show_colors[0] = 1;
+
+    if (!get_int ("DRG_SHOW_COLORS_1", &DRG_show_colors[1], 0, 1, 1))
+        DRG_show_colors[1] = 1;
+
+    if (!get_int ("DRG_SHOW_COLORS_2", &DRG_show_colors[2], 0, 1, 1))
+        DRG_show_colors[2] = 1;
+
+    if (!get_int ("DRG_SHOW_COLORS_3", &DRG_show_colors[3], 0, 1, 1))
+        DRG_show_colors[3] = 1;
+
+    if (!get_int ("DRG_SHOW_COLORS_4", &DRG_show_colors[4], 0, 1, 1))
+        DRG_show_colors[4] = 1;
+
+    if (!get_int ("DRG_SHOW_COLORS_5", &DRG_show_colors[5], 0, 1, 1))
+        DRG_show_colors[5] = 1;
+
+    if (!get_int ("DRG_SHOW_COLORS_6", &DRG_show_colors[6], 0, 1, 1))
+        DRG_show_colors[6] = 1;
+
+    if (!get_int ("DRG_SHOW_COLORS_7", &DRG_show_colors[7], 0, 1, 1))
+        DRG_show_colors[7] = 1;
+
+    if (!get_int ("DRG_SHOW_COLORS_8", &DRG_show_colors[8], 0, 1, 1))
+        DRG_show_colors[8] = 1;
+
+    if (!get_int ("DRG_SHOW_COLORS_9", &DRG_show_colors[9], 0, 1, 1))
+        DRG_show_colors[9] = 1;
+
+    if (!get_int ("DRG_SHOW_COLORS_10", &DRG_show_colors[10], 0, 1, 1))
+        DRG_show_colors[10] = 1;
+
+    if (!get_int ("DRG_SHOW_COLORS_11", &DRG_show_colors[11], 0, 1, 1))
+        DRG_show_colors[11] = 1;
+
+    if (!get_int ("DRG_SHOW_COLORS_12", &DRG_show_colors[12], 0, 1, 1))
+        DRG_show_colors[12] = 1;
+#endif  // HAVE_LIBGEOTIFF
+
     // filter values
     // NOT SAVED: Select_.none
     if (!get_int ("DISPLAY_MY_STATION", &Select_.mine, 0, 1, 1))
@@ -1332,8 +1414,8 @@ void load_data_or_default(void) {
 
 
 
-    if (!get_int ("DISPLAY_UNITS_ENGLISH", &units_english_metric, 0, 1, 0))
-        units_english_metric = 0;
+    if (!get_int ("DISPLAY_UNITS_ENGLISH", &english_units, 0, 1, 0))
+        english_units = 0;
 
     if (!get_int ("DISPLAY_DIST_BEAR_STATUS", &do_dbstatus, 0, 1, 0))
         do_dbstatus = 0;
@@ -1507,6 +1589,14 @@ void load_data_or_default(void) {
         if (!get_int (name, &devices[i].fullduplex, 0, 1, 0))
             devices[i].fullduplex = 0;
 
+        xastir_snprintf(name,
+            sizeof(name),
+            "%s",
+            name_temp);
+        strncat (name, "TNC_INIT_KISSMODE", sizeof(name) - strlen(name));
+        if (!get_int (name, &devices[i].init_kiss, 0, 1, 0))
+            devices[i].init_kiss = 0;
+
         xastir_snprintf(name,
             sizeof(name),
             "%s",
@@ -1846,7 +1936,7 @@ void load_data_or_default(void) {
     if(!get_int("VIEW_MESSAGE_RANGE", &vm_range,0,99999,0))
         vm_range=0;
 
-    if(!get_int("VIEW_MESSAGE_LIMIT", &view_message_limit,0,99999,0))
+    if(!get_int("VIEW_MESSAGE_LIMIT", &view_message_limit,10000,99999,10000))
         view_message_limit = 10000;
 
 
